#ifndef ASSOC_H
#define ASSOC_H

#include <QtCore>
#include "evaxmldoc.h"

#define UNLERNBAR   200000      // Kosten fuer nicht lernbare Dinge

// Assocs stellen Beziehungen (Assoziationen, daher "assoc") zwischen Lernstoffen her. 
// eAssocs gibt an welcher Art die Beziehung ist.
enum AssocTypen {
    FORBIDDEN,      // sWert ist unlernbar
    ALLOWED,        // alle vom lernstoffTyp iWert sind unlernbar, ausser sWert (davon sind mehrere moeglich)

    // die Prereq verlangen Voraussetzungen damit der Lernstoff gelernt werden darf
    STATPREREQ,     // sWert ist eine Base-Stat, sie muss groesser oder gleich iWert sein, bzw. kleiner abs(iWert) wenn iWert < 0
    LERNPREREQ,     // sWert muss bekannt sein um Lernstoff zu lernen (Waffe -> Waffengruppe), bei Zaubern sind iWert die LE
    GROUPPREREQ,    // sWert muss in klasse()->gruppen enthalten sein

    // bei Erschaffung relevant
    TYPISCH,        // sWert ist eine Liste von typischen Zaubern oder Fertigkeiten mit EW iWert (nur ein EW fuer alle)
    GRUPPE,         // sWert ist eine Zauber- oder Fertigkeiten-Gruppe, iWert die LE

    LE_KOSTEN,      // sWert ist eine Zauber- oder Fertigkeiten-Gruppe, iWert die LE-Kosten
    TE_KOSTEN,      // dito

    LAST_ASSOC
};

class Assoc {
public:
    AssocTypen  typ;
    QString     sWert;
    int         iWert;

    void write(EvaXmlDoc& doc, QDomElement& parent);
    bool read(const QDomElement& de);

    bool operator==(const Assoc& b) const;
};

class AssocList : public QList<Assoc>  {
public:
    AssocList   assocs(AssocTypen typ) const;   // Unter-Liste mit assocs von diesem typ
    void        addGruppe(const Assoc& a);      // Fertigkeitsgruppen-Assoc sortiert einbauen (Alltag am Anfang, Zauber am Ende etc.)
};

bool s2bool(const QString s);

int findName(const QString& name, const QString* names, int max);

#endif // ASSOC_H
