// Neuen Datenbogen in die Verwaltung aufnehmen:
// 1. Im Designer erstellen
// 2. includen
// 3. zum ADD_BOEGEN - Makro hinzufuegen (letzte Zeile mit '\' verlaengern, neue Zeile dazu)
//    dabei ist die erste Ziffer der Bogen-Satz, die zweite die Nummer des Bogens im Satz
#include "bogen01a.h"
#include "bogen01b.h"
#include "bogen01c.h"

// Template zur Erzeugung einer Funktion die ein neues Datenblatt anlegt aus einer Klasse
template <class T> QWidget* newBogen(QWidget* parent) {
    return new T(parent);
}

#define ADD_BOEGEN  \
    addBogen(1, 0, newBogen<bogen01a>); \
    addBogen(1, 1, newBogen<bogen01b>); \
    addBogen(1, 2, newBogen<bogen01c>);

#include "boegen.h"

#include <qmetaobject.h>
#include <qfile.h>
#include <qdebug.h>

Boegen      gBoegen;

Bogen::Bogen(int satz, int bogen, QWidget* parent) {
    widget = gBoegen.bogen(satz, bogen, parent);
    getLabels(widget);

    QString key = gBoegen.makeKey(satz, bogen);
    action = new QAction(parent);
    action->setObjectName("action_" + key);
    action->setCheckable(true);
    QIcon icon;
    key = ":/EVA/Resources/" + key + ".png";
    if (!QFile(key).exists()) {
        int l = key.length();
        key = key.right(l - key.indexOf('_'));
        key = ":/EVA/Resources/s0" + key;
    }
    if (QFile(key).exists())
        icon.addFile(key);
    else
        icon.addFile(":/EVA/Resources/bogen.png");
    action->setIcon(icon);
    action->setText(widget->windowTitle());
}

Bogen::Bogen(QWidget* w) {
    widget = w;
    action = NULL;
    getLabels(w);
}

Bogen::~Bogen() {
    delete widget;
}

// Hastabelle aufbauen
void Bogen::getLabels(QObject* w) {
    foreach(QObject* o, w->children())
        getLabels(o);
    QString className = w->metaObject()->className();
    QString objName   = w->objectName();
    if (className == "QLabel" && !objName.startsWith("label")
        || className == "QLineEdit" && !objName.contains("line")
        || className == "QPlainTextEdit" && !objName.contains("plain")
        || className == "QSpinBox" && !objName.startsWith("spinbox") ) {
            Q_ASSERT(!hash.keys().contains(objName));
            hash.insert(objName, w);
    }
};

void Bogen::update(const Figur* f) 
{
    QString     val;
    QByteArray  propertyName;
    QObject*    w;
    QString     className;

    foreach (QString widgetName, hash.keys()) {
        w = hash.value(widgetName);
        Q_ASSERT(w != NULL);
        val = f->widgetValue(widgetName);
        className = w->metaObject()->className();
        if (className == "QLabel" || className == "QLineEdit")
            w->setProperty("text", val);
        if (className == "QPlainTextEdit")
            qobject_cast<QPlainTextEdit*>(w)->setPlainText(val);
        else if (QString(w->metaObject()->className()) == "QSpinBox")
            w->setProperty("value", val.toInt());
    }
    widget->update();
}

void Bogen::getPP(Figur* f) {
    foreach(QString widgetName, hash.keys()) {
        if (!(widgetName.contains("_pp") || widgetName.contains("zauber_wirkung")))
            continue;
        QStringList wnl = widgetName.split('_');
        if (wnl.length() < 3)
            continue;
        int i = wnl[2].toInt() - 1;
        Lernstoff* l = NULL;
        if (i < 0)          // vielleicht die Form fertigkeit_raufen_pp (M4) ?
            l = f->gelernt(wnl[1]);
        else {
            QLabel* fl = NULL;
            QString wn = wnl[0] + "_" + wnl[2];
            if (hash.keys().contains(wn))
                fl = qobject_cast<QLabel*>(hash[wn]);
            if (fl != NULL) {
                wn = fl->text();
                if (wn.contains("+"))
                    wn = wn.left(wn.indexOf("+"));
                l = f->gelernt(wn);
            }
            if (l == NULL) {    // war wohl nix, also danach suchen
                int j = -1;
                foreach(Lernstoff* ll, f->gelerntList) {    // const geht nicht
                    if (ll->typ() == FERTIGKEIT || ll->typ() == WAFFENGRUPPE || ll->typ() == ZAUBERGRUPPE)
                        j++;
                    if (j == i) {
                        l = ll;
                        break;
                    }
                }
            }
        }
        if (l == NULL)
            continue;       // return geht hier leider nicht, weil die Widgets nicht sortiert ankommen
        if (widgetName.contains("_pp"))
            l->setPP(hash.value(widgetName)->property("value").toInt());
        else {
            QPlainTextEdit* pte = qobject_cast<QPlainTextEdit*>(hash.value(widgetName));
            if (pte != NULL)
                l->setProperty("Beschreibung", pte->toPlainText());
        }
    }
}

void Bogen::connectPP(const QObject* receiver, const char* member) {
    foreach(QString widgetName, hash.keys()) {
        if (widgetName.contains("_pp")) {
            QSpinBox* pp = qobject_cast<QSpinBox*>(hash[widgetName]);
            pp->connect(pp, SIGNAL(valueChanged(int)), receiver, member);
        }
        if (widgetName.contains("zauber_wirkung")) {
            QPlainTextEdit* pte = qobject_cast<QPlainTextEdit*>(hash[widgetName]);
            pte->connect(pte, SIGNAL(textChanged()), receiver, member);
        }
    }
}


//****************  Boegen  ************************

Boegen::Boegen(void)
{
    lastSatz = 0;
    ADD_BOEGEN
}

QString Boegen::makeKey(int satz, int bogen) {
    return QString("s%0_b%1").arg(satz).arg(bogen);
}

int Boegen::bogenNr(QString key) {
    key.remove(0, key.indexOf("b") + 1);
    bool ok;
    int ret = key.toInt(&ok);
    Q_ASSERT(ok);
    return ret;
}

void Boegen::addBogen(int satz, int bogen, newBogenType newBogen) {
    if (satz > lastSatz)
        lastSatz = satz;
    QString s = makeKey(satz, bogen);
    if (contains(s))
        return;
    insert(s, newBogen);
}

int Boegen::saetze() {
    return lastSatz;
}

int Boegen::boegen(int set) {
    QString s = QString("s%1").arg(set);
    int boegen = 0;
    foreach(QString k, keys()) {
        if (!k.contains(s))
            continue;
        if (bogenNr(k) >= boegen)
            boegen = bogenNr(k) + 1;
    }
    return boegen;
}

QWidget* Boegen::bogen(int satz, int bogen, QWidget* parent) {
    QString key = makeKey(satz, bogen);
    Q_ASSERT(contains(key));
    return value(key)(parent);
}