#pragma once

#include "figur.h"

#include <qwidget.h>
#include <qaction.h>
#include <qmap.h>

class Bogen {
/*
 * Ein Bogen besteht aus einem Widget sowie Verwaltungsinformation dazu. 
 * Diese umfasst eine Hashtabelle mit Zeigern auf das Widget und seine
 * Nachkommen, die dazu verwendet wird, das Widget schnell aus den Properties
 * einer Figur zu aktualisieren.
 */

public:
    Bogen(int s, int b, QWidget* p);            // neuen Bogen anlegen (ueber gBoegen)
    Bogen(QWidget* w);                          // ein existierendes Widget als Bogen aufnehmen
    ~Bogen();

    QWidget*    getWidget() {return widget;};
    QAction*    getAction() {return action;};   // Die Action welche den Bogen anzeigt
    void        update(const Figur *f);         // das Widget mit den Werten der Figur aktualisieren
    void        getPP(Figur* f);                // PP der Lernstoffe der Figur aktualisieren
    void        connectPP(const QObject* receiver, const char* member);

private:
    void getLabels(QObject* w);

    QHash<QString, QObject*>    hash;
    QWidget*                    widget;
    QAction*                    action;
};

// Prototyp fuer Funktionen, welche neue Datenblaetter erzeugen
typedef QWidget* (*newBogenType)(QWidget*);

// Boegen verwaltet eine Map mit Funktionen, die neue Datenblaetter erzeugen
class Boegen : QMap<QString, newBogenType> {
public:
    Boegen();
    
    int         saetze();                       // wieviele Saetze von Boegen gibt es?
    int         boegen(int satz);               // wieviele Boegen in diesem Satz?

    void        addBogen(int satz, int bogen, newBogenType newBogen);   // Funktion in die Map eintragen
    QWidget*    bogen(int satz, int bogen, QWidget* parent);    // eine der new-Funktionen aufrufen
    QString     makeKey(int satz, int bogen);   // Key (in der Hashtabelle) aus Satz und Bogen basteln

private:
    int         bogenNr(QString key);           // Bogen-Nummer aus key extrahieren
    int         lastSatz;                       // letzter Bogen-Satz
};

extern Boegen   gBoegen;    // globale Liste mit Figurenboegen