#include "ding.h"
#include <QtXml>
#include <QMessageBox>

float stof(const QString s) {
    bool ok;
    float ret = s.toFloat(&ok);
    Q_ASSERT(ok);
    return ret;
}

Ding::Ding(const Ding& d) {
    for (int i = 0; i < metaObject()->propertyCount(); i++) {
        QMetaProperty prop = metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        setProperty(prop.name(), d.property(prop.name()));
    }
}

Ding& Ding::operator=(const Ding& rhs) {
    for (int i = 0; i < metaObject()->propertyCount(); i++) {
        QMetaProperty prop = metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        setProperty(prop.name(), rhs.property(prop.name()));
    }
    return *this;
}

float Ding::fgewicht() const {
    return stof(gewicht());
}

float Ding::fpreis() const {
    return stof(preis());
}

void Ding::setGewicht(const QString newGewicht) {
    m_gewicht = (newGewicht == "") ? "0.0" : newGewicht;
    float g = stof(m_gewicht);
    m_gewicht = "%1";
    m_gewicht = m_gewicht.arg(g, 8, 'f', 3);
}

void Ding::setPreis(const QString newPreis) {
    if (newPreis == "")
        m_preis = "0.0";
    else
        m_preis = newPreis;
    stof(preis());
}

QStringList Ding::list() const {
    QStringList l;
    l << name() << gruppe() << gewicht() << preis() << ort() << desc();
    return l;
}

//************* Guthaben

Guthaben::Guthaben() {
    m_EP = 0;
    m_ES = 0;
    m_gold = 0;
    m_datum = QDateTime::currentDateTime().toTime_t();
}

Guthaben::Guthaben(const Guthaben& g) {
    for (int i = 0; i < metaObject()->propertyCount(); i++) {
        QMetaProperty prop = metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        setProperty(prop.name(), g.property(prop.name()));
    }
}

Guthaben& Guthaben::operator=(const Guthaben& rhs) {
    for (int i = 0; i < metaObject()->propertyCount(); i++) {
        QMetaProperty prop = metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        setProperty(prop.name(), rhs.property(prop.name()));
    }
    return *this;
}

//************* DingList

void xmlWrite(const QObject& obj, QDomElement& e) {
    for (int i = 0; i < obj.metaObject()->propertyCount(); i++) {
        QMetaProperty prop = obj.metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        e.setAttribute(prop.name(), obj.property(prop.name()).toString());
    }
}

bool xmlRead(QObject& obj, const QDomElement& e) {
    for (int i = 0; i < obj.metaObject()->propertyCount(); i++) {
        QMetaProperty prop = obj.metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        obj.setProperty(prop.name(), e.attribute(prop.name()));
    }
    return true;
}

void DingList::addDing(const QString& name) {
    Ding d = Dinge::d(name);
    append(d);
}

QStringList DingList::dingNamen() const {
    QStringList ret;
    foreach (Ding g, *this)
        ret.append(g.name());
    return ret;
}

QString DingList::widgetValue(const QString& widgetName) const {
    QStringList wnl = widgetName.split('_');
    if (wnl.count() != 3)
        return "";
    int i = wnl[2].toInt();
    if (i < 1 || count() < i)
        return "";
    if (widgetName.contains("item"))
        i = 1 - i;  // umkehren damit der Trick einfacher wird (eigentlich ist hier i-1 gemeint)
    else if (widgetName.contains("magic"))
        for (int j = 0; j < count(); j++)       // magische Dinge durchzaehlen
            if (at(j).desc().length() > 1)      // das sind die mit Kommentar versehenen
                if (--i < 1) {                  // wenn wir das i-te finden
                    i = -j;                     // dessen negativen Index merken
                    break;
                }
    if (i > 0)                                  // wenn es kein i-tes magisches Item gab bleibt i positiv
        return "";
    Ding d = at(-i);                            // jetzt mit -i indizieren
    QByteArray pn = wnl[1].toLatin1();
    if (d.metaObject()->indexOfProperty(pn) > -1)
        return d.property(pn).toString();
    return "";
}

void DingList::write(EvaXmlDoc& doc, QDomElement& root) const {
    foreach (Ding d, *this) {
        QDomElement   de = doc.createElement("ding");
        root.appendChild(de);
        xmlWrite(d, de);
    }
}

bool DingList::read(QDomElement& de) {

    if (de.tagName() != "ding")
        return false;
    Ding d;
    if (!xmlRead(d, de))
        return false;
    append(d);
    return true;
}

//************* GuthabenList

void GuthabenList::append(const Guthaben& g) {
    QList<Guthaben>::append(g);
    int len = length();
    if (len < 3)    // die allererste Einnahme (Geld bei Erschaffung) lassen wir auf jeden Fall stehen
        return;
    // Einnahmen und Ausgaben "kumulieren" (aufeinander folgende Transaktionen in dieselbe Richtung zusammenfassen)
    if (    (at(len-1).EP() >= at(len-2).EP()       // die neue Zeile hat groessere EP als die alte
         &&  at(len-2).EP() >= at(len-3).EP() )     // die alte hat groessere EP als die davor
    || (     at(len-1).EP() < at(len-2).EP()        // die neue Zeile hat weniger EP als die alte
         &&  at(len-2).EP() < at(len-3).EP() )  )   // die alte hat weniger EP als die davor
        removeAt(len-2);  // dann die vorletzte Zeile loeschen, denn es wurde 2x hintereinander EP eingenommen oder ausgegeben
}

QString GuthabenList::widgetValue(const QString& widgetName) const {
    if (!widgetName.contains("eptab"))
        return "";
    QStringList wnl = widgetName.split('_');
    if (wnl.count() != 3)
        return "";
    int i = wnl[2].toInt();
    if (1 > i || i > count())
        return "";
    Guthaben g = at(i - 1);
    QByteArray pn = wnl[1].toLatin1();
    if (!g.property(pn).isValid())
        return "";
    if (wnl[1].contains("datum")) {
        if (g.datum() == 0)
            return "";
        QDateTime date;
        date.setTime_t(g.datum());
        return date.toString("dd.MM.yy");
    } else
        return g.property(pn).toString();
}

Guthaben GuthabenList::val() const {
    if (count() < 1)
        return Guthaben();
    return at(count() - 1);
}

void GuthabenList::write(EvaXmlDoc& doc, QDomElement& root) const {

    foreach (Guthaben g, *this) {
        QDomElement   ge = doc.createElement("guthaben");
        root.appendChild(ge);
        xmlWrite(g, ge);
    }
}

bool GuthabenList::read(QDomElement& de) {

    if (de.tagName() != "guthaben")
        return false;
    Guthaben g;
    if (!xmlRead(g, de))
        return false;
    this->append(g);
    return true;
}

//************** Dinge

DingList* Dinge::dl = NULL;

QStringList  Dinge::gewichtNamen = QStringList() << "0.000"
    << "0.010" << "0.025" << "0.050"  << "0.100"  << "0.250"  << "0.500" 
    << "1.000" << "1.500" << "2.000"  << "2.500"  << "3.000"  << "4.000" 
    << "5.000" << "7.500" << "10.000" << "15.000" << "20.000" << "25.000";

QStringList Dinge::ortNamen = QStringList() << "egal"
    << fl("Krper")   << fl("Rcken") << "Schulter" << "Arm" << "Bein" << "Kopf"
    << "Rucksack" << "Sack"   << "Kiste" << fl("Kstchen") << fl("Schriftrollenhlle")
    << "Gewand"   << "Hose"   << fl("Grtel") << fl("rmel") << "Schuh"
    << "Transporttier"  << "Unterkunft" << "Zuhause";

QStringList Dinge::gruppenNamen;

Dinge::~Dinge() {
}

void Dinge::write() {
    EvaXmlDoc doc("dinge");
    dl->write(doc, doc.root);
    doc.write("dinge.xml");
}

DingList* Dinge::d() {
    if (dl == NULL) {
        dl  = new DingList();
        EvaXmlDoc doc("");
        if (!doc.read("dinge.xml"))
            return dl;
        if (doc.root.tagName() != "dinge")
            return dl;
        for(QDomElement n = doc.root.firstChildElement(); !n.isNull(); n = n.nextSiblingElement())
            dl->read(n);
        foreach(Ding dd, *dl)
            gruppenNamen.append(dd.gruppe());
        gruppenNamen.removeDuplicates();
        gruppenNamen.sort();
    }
    return dl;
}

Ding Dinge::d(const QString& name) {
    QString n = name;
    if (n.contains("*"))
        n = n.remove('*');
    foreach(Ding dd, *d())
        if (n == dd.name()) {
            if (name.contains("*") && dd.desc() == "")
                if (dd.gruppe().contains("Waffe"))
                    if (dd.gruppe().contains("Verteidigung"))
                        dd.setDesc("magisch (+1)");
                    else
                        dd.setDesc("magisch (+0/+1)");
                else
                    dd.setDesc("magisch");
            return dd;
        }
    return Ding();
}

void Dinge::append(const Ding& ding) {
    if (dl == NULL)
        return;
    foreach(Ding dd, *dl)
        if (dd.name() == ding.name())
            return;
    dl->append(ding);
    if (!gruppenNamen.contains(ding.gruppe())) {
        gruppenNamen.append(ding.gruppe());
        gruppenNamen.sort();
    }
}
