#ifndef DING_H
#define DING_H

#include <QtCore>
#include <QtXml>
#include "lernstoff.h"

class Ding : public QObject {
    Q_OBJECT

private:
    Q_PROPERTY(QString name     READ name     WRITE setName     );
    Q_PROPERTY(QString gruppe   READ gruppe   WRITE setGruppe   );
    Q_PROPERTY(QString gewicht  READ gewicht  WRITE setGewicht  );
    Q_PROPERTY(QString preis    READ preis    WRITE setPreis    );
    Q_PROPERTY(QString ort      READ ort      WRITE setOrt      );
    Q_PROPERTY(QString desc     READ desc     WRITE setDesc     );

public:
    Ding() {};
    Ding(const Ding& d);

    Ding& operator=(const Ding&);

    QString name   () const {return m_name   ;};
    QString gruppe () const {return m_gruppe ;};
    QString gewicht() const {return m_gewicht;};
    QString preis  () const {return m_preis  ;};
    QString ort    () const {return m_ort    ;};
    QString desc   () const {return m_desc   ;};
    float  fgewicht() const;
    float  fpreis  () const;
    QStringList list() const;       // name, gruppe, gewicht etc. als Liste fuer die TreeWidgets

    void setName   (const QString name   ) {m_name   = name   ;};
    void setGruppe (const QString gruppe ) {m_gruppe = gruppe ;};
    void setGewicht(const QString gewicht);
    void setPreis  (const QString preis  );
    void setOrt    (const QString ort    ) {m_ort    = ort    ;};
    void setDesc   (const QString desc   ) {m_desc   = desc   ;};

private:
    QString     m_name;
    QString     m_gruppe;
    QString     m_gewicht;
    QString     m_preis;
    QString     m_ort;
    QString     m_desc;
};

class Guthaben : public QObject {
    Q_OBJECT

public:
    Guthaben();
    Guthaben(const Guthaben& g);

    Guthaben& operator=(const Guthaben& rhs);

private:
    Q_PROPERTY(int EP       READ EP     WRITE setEP);
    Q_PROPERTY(int ES       READ ES     WRITE setES);
    Q_PROPERTY(int gold     READ gold   WRITE setGold);
    Q_PROPERTY(int datum    READ datum  WRITE setDatum);

public:
    int EP()    const {return m_EP;  };
    int ES()    const {return m_ES;  };
    int gold()  const {return m_gold; };
    int datum() const {return m_datum;};

    void setEP(int EP)          {m_EP = EP;};
    void setES(int ES)          {m_ES = ES;};
    void setGold(int gold)      {m_gold = gold;};
    void setDatum(int datum)    {m_datum = datum;};

private:
    int     m_EP;
    int     m_ES;
    int     m_gold;
    int     m_datum;
};

class DingList : public QList<Ding> {
public:
    void        addDing(const QString& name);       // Ding aus der globalen Liste (Dinge) holen und anhaengen
    QStringList dingNamen() const;                  // Namen der Dinge in dieser Liste als QStrinList
    QString     widgetValue(const QString& widgetName) const;
    void        write(EvaXmlDoc& doc, QDomElement& root) const;
    bool        read(QDomElement& de);

};

class GuthabenList : public QList<Guthaben> {
public:
    void        append(const Guthaben& g);
    QString     widgetValue(const QString& widgetName) const;
    Guthaben    val() const;                   // derzeitiger Wert (letzter Eintrag)
    void        write(EvaXmlDoc& doc, QDomElement& root) const;
    bool        read(QDomElement& de);
};

// Singleton fuer die globale Dinge-Datenbank
class Dinge {
public:
    static DingList* d();
    static Ding      d(const QString& name);
    static void      append(const Ding& ding);
    static void      write();

    static QStringList  gruppenNamen;
    static QStringList  gewichtNamen;
    static QStringList  ortNamen;

private:
    Dinge() {};
    ~Dinge();
    static DingList* dl;
};


#endif // DING_H
