#include "dlgdingedb.h"
#include <QComboBox>
#include <qstyleditemdelegate.h>
#include <qinputdialog.h>

QHash<QString, QTreeWidgetItem*>   gruppen;    // Hashtabelle mit den Gruppen-Items

// Ein Delegate fuer die Comboboxen im TreeWidget
class CItemDBDelegate : public QStyledItemDelegate {

public:
    CItemDBDelegate(QObject *parent = 0):QStyledItemDelegate(parent) {};
 
    virtual QWidget *createEditor( QWidget *parent, const QStyleOptionViewItem&, const QModelIndex &index ) const {
        QComboBox *cb = new QComboBox(parent);
        int      col = index.column();
        cb->setEditable(col != 1);                            // gruppe ist nicht editable
        if      (col == 1) cb->addItems(Dinge::gruppenNamen);
        else if (col == 2) cb->addItems(Dinge::gewichtNamen);
        else if (col == 4) cb->addItems(Dinge::ortNamen);
        else Q_ASSERT(false);
        return cb;
    };

    virtual void setEditorData ( QWidget *editor, const QModelIndex &index ) const {
        QComboBox *cb = qobject_cast<QComboBox *>(editor);      // combobox holen
        Q_ASSERT(cb != NULL);
        QString text = index.data(Qt::EditRole).toString();     // aktuellen Text holen
        int cbIndex = cb->findText(text);                       // nachschaun ob's den gibt
        if(cbIndex >= 0)
            cb->setCurrentIndex(cbIndex);                       // ja - Liste darauf einstellen
        else if (text.length() > 1) {
            cb->insertItem(0, text);                            // nein - vorne einfuegen
            cb->setCurrentIndex(0);
        }
    };

    virtual void setModelData ( QWidget *editor, QAbstractItemModel *model, const QModelIndex &index ) const {
        QComboBox *cb = qobject_cast<QComboBox *>(editor);      // combobox holen
        Q_ASSERT(cb != NULL);
        QString   text = cb->currentText();
        model->setData(index, text, Qt::EditRole);              // Text eintragen
    };

    virtual QSize sizeHint( const QStyleOptionViewItem &, const QModelIndex&) const {
        QComboBox cb;
        QSize hint = cb.sizeHint();
        return hint;
    }
};  // class CItemDBDelegate


//******************* DlgDingeDb

DlgDingeDb::DlgDingeDb(QWidget *parent)
    : QDialog(parent)
{
    setupUi(this);
    items->clear();
    items->setHeaderLabels(QStringList() << "Name des Gegenstands" << "Gegenstands-Gruppe" << "Gewicht (kg)" << "Preis (GS)" << "Aufbewahrungsort" << "Beschreibung");
    for(int i = 0; i < 6; i++)
        if (i < 2 && i > 3)
            items->header()->setSectionResizeMode (i, QHeaderView::ResizeToContents);
    items->header()->setSectionResizeMode (0, QHeaderView::Stretch);
    items->setItemDelegateForColumn(1, new CItemDBDelegate(items));
    items->setItemDelegateForColumn(2, new CItemDBDelegate(items));
    items->setItemDelegateForColumn(4, new CItemDBDelegate(items));
    Dinge::d();                 // ahem - globale Dinge-Liste initialisieren, es koennte sein dass Dinge::d() noch nie aufgerufen wurde
    on_sortGroups_clicked();
}

void saveDing(QTreeWidgetItem* twi) {
    Ding d;
    d.setName(twi->text(0));
    d.setGruppe(twi->text(1));
    d.setGewicht(twi->text(2));
    d.setPreis(twi->text(3));
    d.setOrt(twi->text(4));
    d.setDesc(twi->text(5));
    Dinge::append(d);           // Dinge::append() achtet darauf, dass es nur ein Ding gleichen Namens geben kann
}

void DlgDingeDb::getDinge() {
    foreach (QTreeWidgetItem* gruppe, gruppen)
        foreach(QTreeWidgetItem* child, gruppe->takeChildren())
            saveDing(child);
    while (items->topLevelItemCount() > 0) {
        QTreeWidgetItem *twi = items->takeTopLevelItem(0);
        if (!gruppen.contains(twi->text(0)))
            saveDing(twi);
    }
}

void DlgDingeDb::addDing(const Ding* d) {
    QTreeWidgetItem* twi;
    if (gruppen.contains(d->gruppe()))
        twi = new QTreeWidgetItem(gruppen[d->gruppe()], d->list());
    else
        twi = new QTreeWidgetItem(items, d->list());

    twi->setFlags(twi->flags() | Qt::ItemIsEditable);
    twi->setTextAlignment(2, Qt::AlignRight | Qt::AlignVCenter);
    twi->setTextAlignment(3, Qt::AlignRight | Qt::AlignVCenter);
}

void DlgDingeDb::accept() {
    getDinge();
    Dinge::write();
    QDialog::accept();
}

void DlgDingeDb::on_addItem_clicked() {
    Ding* d = new Ding();
    d->setName("Neu");
    addDing(d);
}

void DlgDingeDb::on_addGroup_clicked() {
    QString gruppe = QInputDialog::getText(this, "Gruppenname", "Name der neuen Gruppe:");
    if (gruppen.contains(gruppe))
        return;
    gruppen[gruppe] = new QTreeWidgetItem(items, QStringList(gruppe));
    Dinge::gruppenNamen.append(gruppe);
    Dinge::gruppenNamen.removeDuplicates();
    Dinge::gruppenNamen.sort();
}

void DlgDingeDb::on_sortGroups_clicked() {
    getDinge();
    items->clear();
    foreach(QString g, Dinge::gruppenNamen)
        gruppen[g] = new QTreeWidgetItem(items, QStringList(g));
    foreach(Ding d, *Dinge::d())
        addDing(&d);
}
