#include "dlgdingefigur.h"
#include <qstyleditemdelegate.h>
#include <qtreeview.h>
#include <qstandarditemmodel.h>
#include <qlineedit.h>

// Delegate fuer die Comboboxen
class DingeCombo : public QStyledItemDelegate {

public:
    DingeCombo(QObject *parent = 0) : QStyledItemDelegate(parent) {};

    virtual QWidget *createEditor( QWidget *parent, const QStyleOptionViewItem &, const QModelIndex &index ) const {
        // neue Combobox anlegen und mit der richtigen Text-Liste vorbelegen
        QComboBox*cb = new QComboBox(parent);

        int col = index.column();

        cb->setEditable(col != 0);    // hier ist Qt buggy! Mit editable true wird der Baum komisch
        if        (col == 0) {
            QTreeView *tree = new QTreeView(cb);
            tree->setHeaderHidden(true);
            cb->setView(tree);
            QHash<QString, QStandardItem*> comboItems;
            QString gruppe;
            foreach(Ding g, *Dinge::d()) {
                gruppe = g.gruppe();
                if (gruppe == "")
                    cb->addItem(g.name());
                else {
                    if (!comboItems.contains(gruppe)) {
                        comboItems[gruppe] = new QStandardItem(gruppe);
                        comboItems[gruppe]->setSelectable(false);
                        qobject_cast<QStandardItemModel*>(cb->model())->appendRow(comboItems[gruppe]);
                    }
                    comboItems[gruppe]->appendRow(new QStandardItem(g.name()));
                }
            }
        } else if (col == 1)  
            cb->addItems(Dinge::gewichtNamen);
        else if (col == 2)  
            cb->addItems(Dinge::ortNamen);
        else                
            return new QLineEdit(parent);
        return cb;
    };

    virtual void setEditorData ( QWidget *editor, const QModelIndex &index ) const {
        if (index.column() == 3) {
            qobject_cast<QLineEdit*>(editor)->setText(index.data(Qt::EditRole).toString());
            return;
        }
        QComboBox *cb = qobject_cast<QComboBox *>(editor);    // combobox holen
        Q_ASSERT(cb != NULL);
        QString text = index.data(Qt::EditRole).toString();   // aktuellen Text holen
        int cbIndex = cb->findText(text);                     // nachschaun ob's den gibt

        if(cbIndex >= 0)
            cb->setCurrentIndex(cbIndex);                       // ja - Liste darauf einstellen, alles gut
        else if (text.length() > 1) {
            cb->insertItem(0, text);                            // nein - vorne einfuegen, auch gut
            cb->setCurrentIndex(0);
        }
    };

    virtual void setModelData ( QWidget *editor, QAbstractItemModel *model, const QModelIndex &index ) const {
        if (index.column() == 3) {
            model->setData(index, qobject_cast<QLineEdit*>(editor)->text(), Qt::EditRole);
            return;
        }
        QComboBox *cb = qobject_cast<QComboBox *>(editor);    // combobox holen
        Q_ASSERT(cb != NULL);
        QString   text = cb->currentText();
        model->setData(index, text, Qt::EditRole);            // Text eintragen
        if (index.column() == 0) {      // Gegenstand-Name veraendert, mal versuchen Ort und Gewicht einzutragen
            Ding d = Dinge::d(text);
            model->setData(model->index(index.row(), 1), d.gewicht());
            model->setData(model->index(index.row(), 2), d.ort());
        }
    };
};

// Ein TablewidgetItem welches sich einen Zeiger auf ein Ding merkt
class DingTwi : public QTableWidgetItem {
public:
    DingTwi(Ding* d) : QTableWidgetItem(d->name()) {m_ding = d;};

    Ding* ding() {return m_ding;};

private:
    Ding* m_ding;
};


DlgDingeFigur::DlgDingeFigur(Figur* figur, QWidget *parent)
    : QDialog(parent)
{
    setupUi(this);
    m_figur = figur;
    gegenstaende->clear();
    gegenstaende->setHorizontalHeaderLabels(QStringList() << "Gegenstand" << "Gewicht (kg)" << "wo getragen" << "Beschreibung");
    gegenstaende->horizontalHeader()->setSectionResizeMode (0, QHeaderView::Stretch);
    gegenstaende->horizontalHeader()->setSectionResizeMode (2, QHeaderView::Stretch);
    gegenstaende->horizontalHeader()->setSectionResizeMode (3, QHeaderView::Stretch);
    gegenstaende->setItemDelegate(new DingeCombo(gegenstaende));

    foreach(Ding g, figur->dinge) {
        int row = gegenstaende->rowCount();
        gegenstaende->insertRow(row);
        setDing(row, g);
    }
    gegenstaende->resizeColumnsToContents();
}

void DlgDingeFigur::setDing(const int row, Ding& d) {
    gegenstaende->setItem(row, 0, new QTableWidgetItem(d.name()));
    gegenstaende->setItem(row, 1, new QTableWidgetItem(d.gewicht()));
    gegenstaende->setItem(row, 2, new QTableWidgetItem(d.ort()));
    gegenstaende->setItem(row, 3, new QTableWidgetItem(d.desc()));
}

Ding* DlgDingeFigur::getDing(int row) const {
    if (gegenstaende->item(row, 0) == NULL)
        return NULL;
    Ding* g = new Ding();
    g->setName   (gegenstaende->item(row, 0)->text());
    g->setGewicht(gegenstaende->item(row, 1)->text());
    g->setOrt    (gegenstaende->item(row, 2)->text());
    g->setDesc   (gegenstaende->item(row, 3)->text());
    return g;
}

void DlgDingeFigur::accept() {
    m_figur->dinge.clear();
    for(int i = 0; i < gegenstaende->rowCount(); i++) {
        Ding* g = getDing(i);
        if (g == NULL || g->gewicht() == "")
            continue;
        if (g->preis() == "")
            g->setPreis("0.0");
        m_figur->dinge.append(*g);
    }
    m_figur->dirty = true;
    QDialog::accept();
}

void DlgDingeFigur::on_add_clicked() {
    Ding g;
    int row = gegenstaende->rowCount();
    gegenstaende->insertRow(row);
    setDing(row, g);
}

void DlgDingeFigur::on_remove_clicked() {
    int row = gegenstaende->currentRow();
    gegenstaende->removeRow(row);
}

void DlgDingeFigur::on_removeAll_clicked() {
    gegenstaende->clear();
}
