#include "qinputdialog.h"
#include "dlgepausgeben.h"
#include "dlgepmix.h"

DlgEPausgeben::DlgEPausgeben(Figur* f, LernstoffTypen lt, QWidget *parent)
    : QDialog(parent)
{
    setupUi(this);
    lernTyp = lt;
    figur   = f;
    desc->setText("");
    actFertigkeit->setText("");
    actEP->setText("");
    actGeld->setText("");
    spruchrolle->setChecked(false);
    if (lernTyp != ZAUBER)
        spruchrolle->hide();
    else
        spruchrolle->show();
    gutEP->setText  (i2s(figur->guthaben.val().EP()));
    gutGeld->setText(i2s(figur->guthaben.val().gold()));
    gutPP->setText("0");
    QStringList header = QStringList() << gLernstoffTypNamen[lernTyp];
    if (lernTyp == ZAUBER)
        header << "Stufe";
    else
        header << "EW";
    header << "EP" << "Gold";
    fertigkeiten->setHeaderLabels(header);
    fertigkeiten->setColumnWidth(0, 170);
    fertigkeiten->setColumnWidth(1, 60);
    fertigkeiten->setColumnWidth(2, 60);
    fertigkeiten->setColumnWidth(3, 60);
    setList();
}

void DlgEPausgeben::addRow(QTreeWidget* tw, const Lernstoff *l, const QString& EW) {
    QFont*    font;
    QStringList sl = QStringList() 
    << figur->klasse()->kostenName(l, font, false) << EW << i2s(figur->EPKosten(l, 0)) << i2s(figur->geldKosten(l, 0));
    if (figur->isSpezialisierung(l))
        font->setWeight(80);
    QTreeWidgetItem* twi = new QTreeWidgetItem(tw, sl);
    twi->setFont(0, *font);
    delete font;
}

void DlgEPausgeben::setList() {
    fertigkeiten->clear();
    LernList lernbar = figur->lernbar(lernTyp, spruchrolle->isChecked() ? LK_ROLLE : LK_VOLL);
    lernbar.sort(LES_STOFFE);
    foreach(Lernstoff* l, lernbar) {
        QString lernEW = figur->EWStr(l);
        if (lernTyp == ZAUBER)
            lernEW = "";
        if (l->name() == "AP verbessern")
            lernEW = i2s(figur->grad()) + "W3+" + i2s(figur->apBonus());
        addRow(fertigkeiten, l, lernEW);
    }
}

void DlgEPausgeben::on_fertigkeiten_itemSelectionChanged() {
    Lernstoff* l = figur->lernbar(fertigkeiten->currentItem()->text(0));
    if (l != NULL) {
        desc->setText(l->property("Beschreibung").toString());
        QTreeWidgetItem* ci = fertigkeiten->currentItem();
        if (*l == "AP verbessern")
            actFertigkeit->setText(ci->text(0));
        else
            actFertigkeit->setText(ci->text(0) + ci->text(1));
        actEP->setText(ci->text(2));
        actGeld->setText(ci->text(3));
        gutPP->setText(i2s(l->PP()));
    } else {
        desc->setText("");
        actFertigkeit->setText("");
        actEP->setText("");
        actGeld->setText("");
        gutPP->setText("");
    }
    delete l;
}

void DlgEPausgeben::on_lernen_clicked() {
    if (fertigkeiten->currentItem() == NULL)
        return;
    Lernstoff *l = figur->lernbar(fertigkeiten->currentItem()->text(0));
    figur->setKosten(l, spruchrolle->isChecked() ? LK_ROLLE : LK_VOLL);
    DlgEPmix epmix(l, figur, this);
    if (epmix.exec()) {
        if (l->name() == "AP verbessern") {
            bool ok;
            l->setEW(QInputDialog::getInt(this, "Neue AP bestimmen",    // EW missbrauchen um die neuen AP zu transportieren
            "Alter Wert: " + i2s(figur->ausdauer()) + "\n" + i2s(figur->grad()) + "W6+" + i2s(figur->apBonus()) + " wuerfeln:",
            figur->rollAP(), figur->ausdauer(), figur->grad() * 6 + figur->apBonus(), 1, &ok));
            if (!ok)
                return;
        } 
        figur->lerne(l, epmix.actEP->text().toInt(), epmix.actGeld->text().toInt(), epmix.actPP->text().toInt());
        accept();
    }
    delete l;
}

void DlgEPausgeben::on_fertigkeiten_doubleClicked() {
    on_lernen_clicked();
}

void DlgEPausgeben::on_spruchrolle_stateChanged(int) {
    setList();
}
