#include "dlgepmix.h"

DlgEPmix::DlgEPmix(const Lernstoff* ls, Figur* f, QWidget *parent)
    : QDialog(parent)
{
    l = ls;
    figur = f;
    setupUi(this);
    geldEPfaktor = QSettings().value("ep1gold", false).toBool() ? 1 : 10;
    heading->setText(l->name() + figur->EWStr(l) + " lernen");
    // iEP, iPP und iG geben an, wieviel wir von der jeweiligen Sorte haben
    int iEP   = figur->guthaben.val().EP();
    int iG    = figur->guthaben.val().gold();
    int minPP = 0;
    for(minPP = 0; minPP <= l->PP(); minPP++)                // PP Minimum suchen
        if (figur->EPKosten(l, minPP) <= iEP && figur->geldKosten(l, minPP) <= iG)
            break;
    minPP = qMin(minPP, l->PP());
    gutEP->setText(i2s(iEP));
    gutGeld->setText(i2s(iG));
    gutPP->setText(i2s(l->PP()));
    int maxEPKosten = qMin(figur->EPKosten(l, minPP), iEP); // max. ausgebbare EP (sowenig PP oder Geld wie moeglich)
    maxEP->setText(i2s(maxEPKosten));
    sliderEP->setMaximum(maxEPKosten);
    int minEPKosten = figur->EPKosten(l, -1);
    minEPKosten = qMax(minEPKosten / 2, minEPKosten - (iG / geldEPfaktor));  // min. auszugebende EP (soviel wie moeglich Geld und PP)
    sliderEP->setMinimum(minEPKosten);
    int maxG = qMin(figur->geldKosten(l, minPP) + maxEPKosten * geldEPfaktor / 2, iG);// ausgebbares Geld Maximum
    maxGeld->setText(i2s(maxG));
    sliderGeld->setMaximum(maxG / geldEPfaktor);            // Obacht: die Skala vom Geld-Slider sind EP-Aequivalente (d.h. durch den Faktor teilen)
    int h = figur->EPKosten(l, -1);                         // EP-Kosten bei max. PP
    h = qMax(h - iEP, 0) * geldEPfaktor;                    // mindest-Geld das fuer EP ausgegeben werden muss
    h = figur->geldKosten(l, -1) + h;                       // Geld Minimum
    sliderGeld->setMinimum(h / geldEPfaktor);
    sliderPP->setMinimum(minPP);
    h = qMin(l->PP(), l->kosten());                         // PP Maximum
    maxPP->setText(i2s(h));
    sliderPP->setMaximum(h);
    changingSliders = false;
    sliderPP->setValue(h);
    if (h == sliderPP->minimum())
        on_sliderPP_valueChanged(h);                        // sicherstellen dass valueChanged wirklich aufgerufen wird
}

void DlgEPmix::setValues() {
    int ep = sliderEP->value();
    int pp = sliderPP->value();
    int geld = sliderGeld->value() * geldEPfaktor;
    actPP->setText(i2s(pp));
    restPP->setText(i2s(gutPP->text().toInt() - pp));
    actEP->setText(i2s(ep));
    restEP->setText(i2s(gutEP->text().toInt() - ep));
    actGeld->setText(i2s(geld));
    restGeld->setText(i2s(gutGeld->text().toInt() - geld));
    int ld = l->kosten();
    if (l->gelernt())
        ld = (ld - pp) * 2;     // 2 Tage je TE (abzuegl. PP)
    else if (l->typ() == ZAUBER)
        ld *= 4;                // 4 Tage je Zauber-LE
    else
        ld *= 6;                // 6 Tage je Fertigkeits-LE
    ld = qMin(60, ld);          // hoechstens 2 Monate
    lerndauer->setText(i2s(ld) + " Tage");
}

void DlgEPmix::setSlider(QSlider* slider, int val) {
// Geld oder EP wurden auf val gesetzt, slider ist der jeweils andere, der jetzt angepasst werden muss
    int k = figur->EPKosten(l, sliderPP->value()) + figur->geldKosten(l, sliderPP->value()) / geldEPfaktor;
    // k sind die Kosten (EP + Geld)
    slider->setValue(k - val);
}

void DlgEPmix::on_sliderPP_valueChanged(int val) {
    if (changingSliders)
        return;
    changingSliders = true;
    int maxEP = figur->EPKosten(l, val);
    int maxG  = qMin(maxGeld->text().toInt() / geldEPfaktor, maxEP / 2);
    int minG  = figur->geldKosten(l, val)  / geldEPfaktor;
    int minEP = maxEP - maxG + minG;
    sliderGeld->setMinimum(minG);
    sliderGeld->setMaximum(maxG);
    sliderEP->setMinimum(minEP);
    sliderEP->setMaximum(maxEP);

    int diff  = (gutGeld->text().toInt() / geldEPfaktor) - gutEP->text().toInt(); // soviel mehr Geld als EP haben wir
    diff = (diff + minG + maxEP) / 2;               // soviel Geld wuerden wir gerne ausgeben
    diff = qMinMax(minG, maxG, diff);               // soviel koennen wir ausgeben
    sliderGeld->setValue(diff);
    sliderEP->setValue(minG + maxEP - diff);
    changingSliders = false;
    setValues();
}

void DlgEPmix::on_sliderEP_valueChanged(int val) {
    if (changingSliders)
        return;
    changingSliders = true;
    setSlider(sliderGeld, val);
    changingSliders = false;
    setValues();
}

void DlgEPmix::on_sliderGeld_valueChanged(int val) {
    if (changingSliders)
        return;
    changingSliders = true;
    setSlider(sliderEP, val);
    changingSliders = false;
    setValues();
}
