#include "eva.h"

#include "neuwizard.h"
#include "dlgoptionen.h"
#include "dlgdingedb.h"
#include "dlgdingefigur.h"
#include "dlgepvergeben.h"
#include "dlgepausgeben.h"
#include "about.h"

#include <qfiledialog.h>
#include <qprinter.h>
#include <qprintdialog.h>

QList<EVA*> alleFenster;    // Liste aller Hauptfenster (fuer den "alles-schliessen"-Knopf)

EVA::EVA(QString fname) : QMainWindow()
{
    ui.setupUi(this);
    qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));  // random-Generator initialisiern
    alleFenster.append(this);
    figur     = new Figur(fname);
    if (fname.length() > 1)
        figur->read(fname);
    agBoegen  = new QActionGroup(this);
    boegen.clear();
    actSatz   = -1;
    actBogen  = -1;
    boegenSep = ui.tools->insertSeparator(ui.actionOptionen);
    ui.stack->setStyleSheet("background-color: white;");
    redraw();
}

EVA::~EVA()
{
    alleFenster.removeAll(this);
}

void EVA::showDialog(QDialog *d, bool doRedraw) {
    if (d->exec() && doRedraw)
        redraw();
    delete d;
}

//***************** Slots 

void EVA::PPchanged() {
    if (-1 < actBogen && actBogen < boegen.count())
        boegen[actBogen]->getPP(figur);
    else
        Q_ASSERT(false);        // wo soll das herkommen?
    ui.actionFertigkeiten->setEnabled(figur->lernbar(FERTIGKEIT, LK_VOLL).count() > 0);
    ui.actionWaffen->setEnabled(figur->lernbar(WAFFENGRUPPE, LK_VOLL).count() > 0);
    ui.actionZauber->setEnabled(figur->lernbar(ZAUBER, LK_ROLLE).count() > 0);
}

// Bogensatz-Wechsel (Dialog Optionen), Bogen-Wechsel (Menue Ansicht) oder redraw
void EVA::redraw() {
    ui.actionEP_vergeben->setEnabled(figur->alive());
    ui.actionGepaeck->setEnabled(figur->alive());
    if (actSatz != settings.value("BogenSatz", 1).toInt()) {    // Satzwechsel
        actSatz = settings.value("BogenSatz", 1).toInt();
        foreach(QAction* action, agBoegen->actions()) {
            disconnect(action, SIGNAL(triggered()), this, SLOT(redraw()));
            ui.menuAnsicht->removeAction(action);
            ui.tools->removeAction(action);
            agBoegen->removeAction(action);
            delete action;
        }
        while (ui.stack->count() > 0)
            ui.stack->removeWidget(ui.stack->widget(0));
        while (boegen.count() > 0)
            delete boegen.takeAt(0);

        for (int bogen = 0; bogen < gBoegen.boegen(actSatz); bogen++) {
            boegen.append(new Bogen(actSatz, bogen, this));
            ui.stack->addWidget(boegen[bogen]->getWidget());
            QAction* action = boegen[bogen]->getAction();
            agBoegen->addAction(action);
            ui.tools->insertAction(boegenSep, action);
            ui.menuAnsicht->addAction(action);
            connect(action, SIGNAL(triggered()), this, SLOT(redraw()));
            if (bogen == 0)
              action->setChecked(true);
            boegen[bogen]->connectPP(this, SLOT(PPchanged()));
        }
        actBogen = -1;
    }
    QAction* action = agBoegen->checkedAction();
    Q_ASSERT(action != NULL);
    int bogen;
    for(bogen = 0; bogen < ui.stack->count(); bogen++)
        if (action == boegen[bogen]->getAction())
            break;
    Q_ASSERT(bogen < ui.stack->count());
    if (bogen != actBogen) {                                    // Bogenwechsel
        ui.stack->setCurrentIndex(bogen);
        actBogen = bogen;
    }
    boegen[bogen]->update(figur);                               // einfacher redraw
    if (figur->dirty)
        setWindowTitle(figur->name() + " *");
    else
        setWindowTitle(figur->name());
    PPchanged();
}

//***************** Menuepunkte
// Datei
void EVA::on_actionNeu_triggered() {
    if (figur->alive() && figur->name() != "")
        (new EVA())->show();
    else
        showDialog(new NeuWizard(&figur, this), true);
}

void EVA::on_actionOeffnen_triggered() {
    QString cname = QFileDialog::getOpenFileName(this, "Figur lesen", "", "EVA-Charaktere (*.eva)");
    if (cname == "")
        return;
    (new EVA(cname))->show();
    if (!figur->dirty || cname == figur->name() + ".eva")
        close();
}

void EVA::on_actionSpeichern_triggered() {
    if (figur->alive())
        figur->write();
    redraw();
}

void EVA::on_actionDrucken_triggered() {
    QPrinter printer (QPrinter::HighResolution);
    printer.setPageSize(QPrinter::A4);
    printer.setOrientation(QPrinter::Landscape);

    QPrintDialog *pd = new QPrintDialog(&printer, this);
    int firstPage = 1;
    int lastPage = ui.stack->count();
    pd->setMinMax(firstPage, lastPage);
    pd->setOption(QAbstractPrintDialog::PrintCurrentPage);
    if (pd->exec() != QDialog::Accepted)
        return;
    switch (pd->printRange()) {
    case QAbstractPrintDialog::AllPages:
        firstPage--;
        lastPage--;
        break;
    case QAbstractPrintDialog::Selection:
        firstPage = -1;
        lastPage = -1;
        break;
    case QAbstractPrintDialog::PageRange:
        firstPage = pd->fromPage() - 1;
        lastPage = pd->toPage() - 1;
        break;
    case QAbstractPrintDialog::CurrentPage:
        firstPage = ui.stack->currentIndex();
        lastPage  = ui.stack->currentIndex();
        break;
    }
    QPainter painter(&printer);
    double scale = 1000.0;
    for (int i = 0; i < ui.stack->count(); i++) {
        double s = printer.pageRect().width() / ui.stack->widget(i)->width();
        scale = qMin(scale, s);
        s = printer.pageRect().height() / ui.stack->widget(i)->height();
        scale = qMin(scale, s);
    }
    painter.scale(scale, scale);
    for (int i = firstPage; i <= lastPage; i++) {
        ui.stack->widget(i)->render(&painter);
        if (i < lastPage)
            printer.newPage();
    }
}

void EVA::on_actionOptionen_triggered() {
    showDialog(new DlgOptionen(gBoegen.saetze(), this), true);
}

void EVA::on_actionQuit_triggered() {
    QList<EVA*> tmp(alleFenster);    // Kopie anlegen, alleFenster veraendert sich
    foreach(EVA* w, tmp)
        w->close();
}

// Lernen
void EVA::on_actionEP_vergeben_triggered() {
    showDialog(new DlgEPvergeben(figur, this), true);
}

void EVA::on_actionFertigkeiten_triggered() {
    showDialog(new DlgEPausgeben(figur, FERTIGKEIT, this), true);
}

void EVA::on_actionWaffen_triggered() {
    showDialog(new DlgEPausgeben(figur, WAFFENGRUPPE, this), true);
}

void EVA::on_actionZauber_triggered() {
    showDialog(new DlgEPausgeben(figur, ZAUBER, this), true);
}

void EVA::on_actionStat_Editor_triggered() {
}

// Ausruestung
void EVA::on_actionGepaeck_triggered() {
    showDialog(new DlgDingeFigur(figur, this), true);
}

void EVA::on_actionStandard_Ausstattung_triggered() {
    QStringList stdItems = QStringList() << "Rucksack (25kg)" << "5l Wasserschlauch" << "abblendbare Laterne" 
        << fl("0.25 l Lampenl") << "Zweipersonenzelt" << "warme Decke" << "Feuerstein u. Zunder" << "Holzkohle" 
        << "Kochgeschirr" << "Reiseverpflegung (1 Woche)";
    foreach(QString s, stdItems) {
        if (!Dinge::d()->dingNamen().contains(s) || figur->dinge.dingNamen().contains(s))
            continue;
        figur->dinge.append(Dinge::d(s));
        figur->dirty = true;
    }
    redraw();
}

void EVA::on_actionDatenbank_triggered() {
    showDialog(new DlgDingeDb(this), false);
}

// Hilfe
void EVA::on_actionAbout_triggered() {
    showDialog(new About(this), false);
}
