#include "evaxmldoc.h"
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>

// Obacht: die Copyright-Notiz wird als Unterscheidungsmerkmal fuer XML-Dateien verwendet, d.h. EVA liest nur Dateien ein,
// welche genau diese Notiz enthalten
#define COPYRIGHT   fl("copyright  fr MIDGARD (Regeln und Daten): Elsa Franke, Verlag fr F&SF-Spiele, Stelzenberg; Email: vfsf@midgardonline.de")

EvaXmlDoc::EvaXmlDoc(const QString& rootname) : QDomDocument()
{
    if (rootname == "")
        return;
    appendChild(createProcessingInstruction("xml", "version = \"1.0\" encoding = \"ISO-8859-1\""));
    root = createElement(rootname);
    appendChild(root);
    QDomElement copyright = createElement("copyright");
    copyright.setAttribute("notiz", COPYRIGHT);
    root.appendChild(copyright);
}

void EvaXmlDoc::write(const QString& fname) {
    QFile   of(fname);
    if (!of.open(QIODevice::WriteOnly))
        return;
    QTextStream ts(&of);
    ts << toString();
    of.close();
}

bool EvaXmlDoc::read(const QString& fname) {
    QString estr;
    int eline, ecol;

    clear();
    QFile inf(fname);
    if (!inf.open(QIODevice::ReadOnly))
        return false;
    if (!setContent(&inf, &estr, &eline, &ecol)) {
        QMessageBox msg;
        msg.setText(estr + "\nDatei: " + fname + "\nZeile: " + i2s(eline) + " Spalte: " + i2s(ecol));
        msg.exec();
        return false;
    }
    root = documentElement();
    for(QDomElement n = root.firstChildElement(); !n.isNull(); n = n.nextSiblingElement()) {
        if (n.tagName() != "copyright")
            continue;
        if (n.attribute("notiz") == COPYRIGHT)
            return true;
    }
    return false;
}

QString i2s(int i) {
    return QString::number(i);
}

QString fl(const char* s) {
    QString t = QString::fromLatin1(s);
    return t;
};
