#include "figur.h"
#include "dlggradneu.h"

#include <qscriptengine.h>
#include <qinputdialog.h>

Figur::Figur(QString fname) : gelerntList(true) {
    dirty = false;
    clear();
}

Figur::Figur(Figur* f) : gelerntList(true) {
    dirty = true;
    foreach(Lernstoff* l, f->gelerntList)
        gelerntList.append(l);
    foreach(Ding d, f->dinge)
        dinge.append(d);
    foreach(Guthaben g, f->guthaben)
        guthaben.append(g);
    for (int i = 0; i < f->metaObject()->propertyCount(); i++) {
        QMetaProperty prop = f->metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        setProperty(prop.name(), f->property(prop.name()));
    }
}

Figur::~Figur(void)
{
    clear();
}

void Figur::clear() {
    guthaben.clear();
    dinge.clear();
    // Properties initialisieren: alle Strings = "" und alle Ints = 0
    for (int i = 0; i < metaObject()->propertyCount(); i++) {
        QMetaProperty prop = metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        if (QString(prop.typeName()) == "QString")
            setProperty(prop.name(), "");
        if (QString(prop.typeName()) == "int")
            setProperty(prop.name(), 0);
    }
}

int statBonus(int stat) {
    if (stat <  6)  return -2;
    if (stat < 21)  return -1;
    if (stat < 81)  return  0;
    if (stat < 96)  return  1;
                    return  2;
}

int Figur::EW(const Lernstoff* l) const {
    int ret = l->EW();
    if (l->typ() == ANGEBOREN && l->property("EWungelernt").isValid()) {
        if (l->property("EWungelernt").toInt() == 10) {
            ret = 11;
            if (grad() > 1)
                ret = 12 + (grad() / 5);
        }
        if (l->property("EWungelernt").toInt() == 3) {
            ret = 11;
            if (grad() < 11)
                ret = ret + grad() / 2;
            else
                ret = 16 + (grad() - 10) / 5;
        }
        ret = qMin(ret, 18);
    }
    if (l->property("Attribut").isValid()) {
        QString s = l->property("Attribut").toString();
        if (s.length() < 4)
            s = gStatNames[s];
        Q_ASSERT(s != "" && property(s.toLatin1()).isValid());
        ret += statBonus(property(s.toLatin1()).toInt());
    }
    return ret;
}

int Figur::EW(const QString name) const {
    foreach(Lernstoff* l, gelerntList)
        if (*l == name)
            return EW(l);
    Q_ASSERT(false);
    return -1;
}

int Figur::PP(const Lernstoff* l) const {
    return l->PP();                         // todo - PP fuer Zauber-Gruppen einbauen
}

QString Figur::EWStr(const Lernstoff* l) const {
    return itos(EW(l), true, true);
}

LernList Figur::gelernt(LernstoffTypen typ) const {
    LernList ret(false);

    foreach(Lernstoff* l, gelerntList)      // const geht nicht (ret soll modifiyable sein)
        if (l->typ() == typ)
            ret.append(l);
    return ret;
}

Lernstoff* Figur::gelernt(QString name) const {
    foreach(Lernstoff* l, gelerntList)    // const geht nicht (als return value)
        if (*l == name)
            return l;
    return NULL;
}

const Lernstoff* Figur::typGelernt(LernstoffTypen typ) const {
    const Lernstoff* ret = NULL;
    foreach(const Lernstoff* l, gelerntList)
        if (l->typ() == typ) {
            Q_ASSERT(ret == NULL);
            ret = l;
        }
    return ret;
}

const Lernstoff* Figur::typGelernt(const QString typ) const {
    LernstoffTypen lTyp;
    for (lTyp = GOTT; lTyp < LAST_LERNSTOFF; lTyp = (LernstoffTypen)(lTyp + 1))
        if (gLernstoffTypNamen[lTyp].toLower() == typ.toLower())
            break;
    if (lTyp == LAST_LERNSTOFF)
        return NULL;
    else
        return typGelernt(lTyp);
}

Rasse* Figur::rasse() const {
    return (Rasse*) typGelernt(RASSE);
}

Klasse* Figur::klasse() const {
    return (Klasse*)typGelernt(KLASSE);
}

Lernstoff* Figur::lerne(Lernstoff* l) {
    foreach(Lernstoff* gl, gelerntList)     // const geht nicht
        if (*l == *gl) {
            gl->setEW(qMax(gl->EW(), l->EW()));
            return gl;
        } else if (l->typ() < ANGEBOREN && l->typ() == gl->typ())   // Typen < ANGEBOREN sind nur einmal lernbar
            vergesse(gl->name());
    Lernstoff* nl = gelerntList.append(l);
    if (l->zusaetze().count() > 0 && l->zusatz() < 0) {
        QStringList zl = zusaetze(nl);
        bool        ok = zl.count() > 0;
        if (ok) {
            QString s = zl[0];
            if (zl.length() > 1)
                s = QInputDialog::getItem(NULL, "Fertigkeit spezifizieren", nl->name() + ":", zl, 0, false, &ok);
            if (ok)
                nl->setZusatz(nl->zusaetze().indexOf(s));
        }
        if (!ok) {
            vergesse(nl->name());
            return NULL;
        }
    }
    if (nl->typ() == WAFFE || nl->typ() == ZAUBER)
        nl->setEW(0);
    if (isSpezialisierung(nl))
        nl->setEW(2);
    nl->setGelernt(true);
    if (nl->typ() == WAFFENGRUPPE) {            // Waffengruppe gelernt
        foreach(Lernstoff* w, gelerntList) {    // alle Waffen vergessen
            if (w->typ() == WAFFE) {
                gelerntList.removeAll(w);
                delete w;
            }
        }
        foreach(Lernstoff* w, lernbar(WAFFE, LK_NONE))   // alle jetzt lernbaren Waffen lernen
            lerne(w);
    }
    gelerntList.sort(LES_GELERNT);
    return nl;
}

void Figur::lerne(Lernstoff* l, int ep, int gold, int pp) {
    Lernstoff* ll = gelernt(l->name());
    if (ll == NULL) {
        ll = lerne(l);
        if (pp != 0)    // wir haben was neues mit PP gelernt - also PP bei den Voraussetzungen abziehn
            foreach(Assoc a, ll->assocs.assocs(LERNPREREQ)) {
                Lernstoff* p = gelernt(a.sWert);
                if (p != NULL && p->PP() > 0) {
                    int h = qMin(pp, p->PP());
                    p->setPP(p->PP() - h);
                    pp -= h;
                    if (pp < 1)
                        break;
                }
            }
    } else
        ll->setPP(ll->PP() - pp);
    if (ll->name() == "AP verbessern")
        setAusdauer(l->EW());   // wir missbrauchen den EW um die gewuerfelten AP zu transportieren
    else
        ll->setEW(l->EW());     // neuen EW setzen
    if (ep + gold <= 0)
        return;

    Guthaben g = guthaben.val();
    g.setGold(g.gold() - gold);
    g.setEP(g.EP() - ep);
    guthaben.append(g);
}

Lernstoff* Figur::lerne(const QString& name) {
    if (name == "")
        return NULL;
    QString s  = name;
    QString zusatz = "";
    int i = s.indexOf('+');
    if (i > -1)
        s.truncate(i);
    i = s.indexOf(':');
    if (i > -1) {
        zusatz = s.mid(i + 1);
        s.truncate(i);
    }
    if (!Stoffe::keys().contains(lowerUmlaut(s)))
        return NULL;
    Lernstoff* l = Stoffe::l(s);
    i = -1;                 // i wird jetzt der Zusatz-Index, wenn noetig (sonst bleibt i < 0)
    if (zusatz != "") {
        i = l->zusaetze().indexOf(zusatz);
        if (i < 1) {       // Spezial-Zusaetze pruefen (Heima, Heimisch und Muttersprache)
            const Lernstoff* heimat = typGelernt(LAND);
            if (zusatz == "Heimat")
                i = l->zusaetze().indexOf(heimat->name());
            if (zusatz == "Heimisch")
                if (heimat->property("Heimisch").isValid())
                    i = l->zusaetze().indexOf(heimat->property("Heimisch").toString());
                else {
                    s = QInputDialog::getItem(NULL, "", fl("Gelndeform"), l->zusaetze());
                    i = l->zusaetze().indexOf(s);
                    if (i < 0)      // "Abbrechen" gibt's hier nicht, nehmen wir halt die erste
                        i = 0;
                }
            if (zusatz == "Muttersprache") {
                QStringList ml = muttersprachen();
                Q_ASSERT(ml.count() > 0);
                if (ml.count() > 1)
                    s = QInputDialog::getItem(NULL, "", "Muttersprache", ml);
                else
                    s = ml[0];
                i = l->zusaetze().indexOf(s);
            }
        }
        Q_ASSERT(i > -1);
    }
    if (name.contains("+"))
        l->setEW(name.mid(name.indexOf('+') + 1).toInt());
    l->setZusatz(i);
    Lernstoff* ret = lerne(l);
    l->setZusatz(-1);
    return ret;
}

void Figur::vergesse(const QString name) {
    if (name.length() < 2)
        return;
    foreach(Lernstoff* l, gelerntList)      // const geht nicht
        if (*l == name) {
            gelerntList.removeAll(l);
            delete l;
        }
}

bool Figur::alive() const
{
    return (name() != "") || (gelerntList.length() > 0);
}

bool Figur::lernbar(Lernstoff* l, eLernkostentyp kosten) const {
    int         allowedState[LAST_LERNSTOFF];   // pro Lernstofftyp: 0 - keine ALLOWED gefunden, 1 - ALLOWED gefunden, aber no match, 2 - ALLOWED match
    if (l == NULL || !alive())
        return false;
    foreach(const Lernstoff* al, gelerntList)
        if (*al == *l) {                    // bereits gelernter Stoff
            if (l->typ() == ZAUBER)
                return false;               // Zauber nicht nochmal lernbar
            l->setGelernt(true);
            if (l->name() == "AP verbessern")
                l->setEW(grad());
            else {
                if (l->typ() == ANGEBOREN || l->EW() > 17)
                    return false;
                l->setEW(al->EW() + 1);     // EW steigern
            }
            l->setPP(al->PP());
        } else {
            memset(allowedState, 0, sizeof(allowedState));
            foreach(Assoc a, al->assocs) {          // die assocs die an den bererits gelernten Stoffen haengen pruefen
                if (a.typ == FORBIDDEN && a.sWert == l->name())
                    return false;
                if (a.typ == ALLOWED && a.sWert == l->name()) {
                    allowedState[a.iWert] = 2;
                }
                if (a.typ == ALLOWED && a.iWert == l->typ() && allowedState[a.iWert] < 2)
                    allowedState[a.iWert] = 1;      // an einem gelernten Stoff haengt eine Allowed-Assoc, die diesen Typ erlaubt, aber nicht genau diesen Stoff
            }
            if (allowedState[l->typ()] == 1)
                return false;
        }
    memset(allowedState, 0, sizeof(allowedState));
    foreach(Assoc a, l->assocs) {   // jetzt die assocs am Lernstoff
        switch (a.typ) {
        case FORBIDDEN:             // an l haengt eine Forbidden-assoc
            if (gelernt(a.sWert))   // wir kennen den Stoff der dadurch
                return false;       // verboten wuerde -> wir koennen l nicht lernen
            break;
        case ALLOWED:
            if (l->typ() == KLASSE && rasse()->name() != "Mensch") {
                // wir ueberpruefen ob diese Figur eine Klasse lernen darf. Bei nicht-Menschen muessen wir die Rasse nehmen:
                foreach(Assoc b, rasse()->assocs)
                    if (b.sWert == l->name() && b.typ == ALLOWED)
                        allowedState[a.iWert] = 2;
            } else if (gelernt(a.sWert))
                allowedState[a.iWert] = 2;
            else if (allowedState[a.iWert] == 0)
                allowedState[a.iWert] = 1;
            break;
        case STATPREREQ: {
            QSettings settings;
            if (!settings.value("Eigenschaften").toBool())
                break;
            int statVal = property(a.sWert.toLatin1()).toInt();
            int lim = abs(a.iWert);
            if (settings.value("Eigenschaften31").toBool())
                lim = qMin(lim, 31);
            if ((a.iWert < 0 && statVal > lim)     // entweder zu huebsch
            ||  (a.iWert > 0 && statVal < lim))    // oder nicht huebsch genug
                return false;
            break;
        }
        case GROUPPREREQ: {
            if (!klasse()->property(a.sWert.toLatin1()).toBool())
                return false;
            break;
        }
        case LERNPREREQ: {
            Lernstoff* p = gelernt(a.sWert);
            if (p == NULL)
                return false;
            if (l->typ() == WAFFE)
                l->setEW(p->EW());
        }
        default: 
            break;
        }
    }
    for(int i = 0; i < LAST_LERNSTOFF; i++)
        if (allowedState[i] == 1)
            return false;
    if (kosten > LK_NONE) {
        setKosten(l, kosten);
        int gk  = geldKosten(l, -1);            // Geldkosten (max. PP)
        int epk = EPKosten(l, -1);              // EP-Kosten (max. PP)
        int gg  = guthaben.val().gold();        // Geldguthaben
        int geldEPfaktor = QSettings().value("ep1gold", false).toBool() ? 1 : 10;
        if (gk > gg)    
            return false;                       // zuwenig Geld
        if (epk > guthaben.val().EP() + qMin(epk / 2, (gg - gk) / geldEPfaktor))
            return false;                       // zuwenig EP
    }
    return l->zusaetze().count() < 1 || zusaetze(l).count() > 0;  // entweder keine Zusaetze oder mindestens einen lernbaren
}

LernList Figur::lernbar(LernstoffTypen typ, eLernkostentyp kosten) const {
    LernList ret(true);
    foreach(Lernstoff* l, Stoffe::l(typ)) {
        if (lernbar(l, kosten) && !ret.contains(l))
            ret.append(l);
        if (l->zusaetze().count() > 0)              // Bei Stoffen mit Zusaetzen pruefen ob wir 
            foreach(Lernstoff* gl, gelerntList)     // einen davon schon gelernt haben
                if ( gl->name().startsWith(l->name())
                &&   lernbar(gl, kosten) 
                &&  !ret.contains(gl) )             // und evtl. verbessern koennen
                    ret.append(gl);
    }
    if (typ == FERTIGKEIT) {    // jetzt noch die lernbaren angeborenen dranhaengen (Abwehr)
        foreach(Lernstoff* l, gelerntList) {
            if (l->typ() != ANGEBOREN)
                continue;
            if (lernbar(l, kosten))
                ret.append(l);
        }
    }
    return ret;
}

Lernstoff* Figur::lernbar(const QString& name) const {
    QString ln = name;
    int ic = ln.indexOf(':');
    if (ic > 0)
        ln = ln.left(ln.indexOf(':'));
    Lernstoff *l = Stoffe::l(ln);
    if (l == NULL)
        return NULL;
    l = l->clone();
    if (ic > 0)
        l->setZusatz(l->zusaetze().indexOf(name.mid(ic + 1)));
    foreach(const Lernstoff* gl, gelerntList) {
        if (*gl != *l) 
            continue;
        if (l->typ() == ZAUBER) {       // gelernten Zauber gefunden - also nicht lernbar
            delete l;
            return NULL;
        }
        l->setPP(gl->PP());
        l->setEW(gl->EW() + 1);
        l->setGelernt(true);
        return l;
    }
    if (l->typ() == ZAUBER) {
        int pp = 0;
        foreach(Assoc a, l->assocs.assocs(LERNPREREQ)) {
            Lernstoff* p = gelernt(a.sWert);
            if (p != NULL) {
                pp += p->PP();
                if (isSpezialisierung(l))
                    l->setEW(2);
            }
        }
        l->setPP(pp);
    }
    return l;
}

QStringList Figur::zusaetze(Lernstoff* l) const {
    QStringList zus;
    QStringList allowed;
    QString     lName;

    zus = l->zusaetze();
    if (zus.count() < 1)
        return allowed;
    lName = l->key();
    foreach(QString z, zus)
        foreach(const Lernstoff* ls, gelerntList)
            if (ls->name().contains(z) && ls->key().contains(lName))
                zus.removeAll(z);
    if (lName == "schreiben") {
        foreach(const Lernstoff* l, gelerntList)
            if (l->name().contains("prache"))
                allowed.append(l->zusaetze()[l->zusatz()]);
    } else
        allowed = zus;
    QStringList ret;
    foreach(QString s, allowed)
        if (zus.contains(s) && !gelernt(lName + ":" + s))
            ret.append(s);
    qSort(ret);
    return ret;
}

void Figur::setKosten(Lernstoff* l, eLernkostentyp kosten) const {
    l->setKosten(kosten);
}

int Figur::EPKosten(const Lernstoff* l, int pp) const {
    int k = l->kosten();                                    // setKosten muss bereits aufgerufen worden sein
    int f = UNLERNBAR;
    if (l->name() == "AP verbessern") {
        if (grad() < 6) return 50;
        if (grad() < 11) return 100;
        if (grad() < 21) return 500;
        return 1000;
    }
    if (pp < 0)
        pp = PP(l);
    pp = qMin(k, pp);                                       // pp sind jetzt die PP, die ausgegeben werden sollen und koennen
    foreach(Assoc a, l->assocs)
        if (a.typ == LE_KOSTEN
        ||  (l->typ() == ZAUBER && a.typ == LERNPREREQ))
            f = qMin(f, klasse()->EPkosten[a.sWert]);       // den niedrigsten Faktor suchen
    if (l->typ() == ZAUBER && isSpezialisierung(l))
        f = qMin(f, 30);
    if (!l->gelernt()) {                                    // bei LE 
        if (rasse()->property("LEKosten").isValid())        // die Extrakosten fuer Rasse dazu
            k += rasse()->property("LEKosten").toInt();
        if (l->typ() != ZAUBER)
            f *= 3;                                         // bei nicht-Zaubern verdreifachen
        else
            k -= pp;                                        // bei Zaubern verfuegbare PP abziehen
    } else
        k -= pp;                                            // bei TE verfuegbare PP abziehen
    if (l->m_lernkostentyp == LK_ROLLE)
        f /= 3;
    return k * f;
}

int Figur::geldKosten(const Lernstoff* l, int pp) const {
    if (l->name() == "AP verbessern") {
        if (grad() < 6) return 50;
        if (grad() < 11) return 100;
        if (grad() < 21) return 500;
        return 1000;
    }
    if (l->m_lernkostentyp == LK_ROLLE)
        return 20;
    int k = l->kosten();
    if (pp < 0)
        pp = PP(l);
    pp = qMin(k, pp);
    if (l->gelernt()) {
        k -= pp;
        return k * 20;
    }
    if (l->typ() == ZAUBER) {
        k -= pp;
        return k * 100;
    }
    return k * 200;
}

QStringList Figur::muttersprachen() const {
    QStringList ret = typGelernt(LAND)->landesSprachen();
    if (rasse()->property("Sprache").isValid()) {       // Fuer Rassen mit eigener Sprache ist diese automatisch die Muttersprache
        ret.clear();
        ret.append(rasse()->property("Sprache").toString());
    }
    return ret;
}

QStringList Figur::zweitsprachen() const {
    QStringList ret;
    const Lernstoff* heimat = typGelernt(LAND);
    const Lernstoff* gast   = Stoffe::l(gastland());
    // Verkehrssprachen des Gastlandes
    if (gast->property("Verkehrssprache").isValid())
        ret = gast->property("Verkehrssprache").toString().split(',');
    // Verkehrssprachen des Heimatlandes
    if (heimat->property("Verkehrssprache").isValid()) {
        foreach(QString s, heimat->property("Verkehrssprache").toString().split(','))
            if (!ret.contains(s))
                ret.append(s);
    }
    // Landessprachen des Gastlandes
    foreach(QString s, gast->landesSprachen())
        if (!ret.contains(s))
            ret.append(s);
    return ret;
}

bool Figur::isSpezialisierung(const Lernstoff* l) const {
    if (spezialisierung().length() < 2)
        return false;
    if (l->typ() == ZAUBER) {
        foreach(Assoc a, l->assocs)
            if (a.typ == LERNPREREQ && a.sWert == spezialisierung())
                return true;
        return false;
    } if (l->typ() == WAFFE)
        if (spezialisierung().contains(l->property("Ding").toString().split(',')[0]))   // muahaha (Spezialwaffe gilt nur fuer das erste Ding)
            return true;
    else 
        return spezialisierung().split(',').contains(l->name());
    return false;
}

int Figur::rollStat(const QString statName, bool zweimal, int numdice, int type) {
    int ret   = roll(numdice, type);
    int limit = rasse()->statLimit(statName);
    if (zweimal && limit >= 0)
        ret = qMax(roll(numdice, type), ret);
    while ( (limit > 0 && (ret < limit))            // nachwuerfeln bis das Ergebnis passt
       ||   (limit < 0 && (ret > abs(limit))) )
        ret = roll(numdice, type);
    return ret;
}

void Figur::rollBaseStats() {
    setStaerke(rollStat("St"));
    setGeschicklichkeit(rollStat("Ge"));
    setGewandheit(rollStat("Gw"));
    setKonstitution(rollStat("Ko"));
    setIntelligenz(rollStat("In"));
    setZaubertalent(rollStat("Zt"));
}

void Figur::rollDerivedStats() {
    int n;

    n = rollStat("pA", false) + 4 * intelligenz() / 10 - 20;
    setAusstrahlung(n);
    n = rollStat("Wk", false) + 2 * (konstitution() / 10 + intelligenz() / 10) - 20;
    setWillenskraft(n);
    setAussehen(rollStat("Au", false));

    setBewegungsweite(rasse()->rollProp("Bewegungsweite"));
    setGroesse(rasse()->rollProp("Groesse")   + staerke() / 10);
    setGewicht(rasse()->rollProp("Gewicht") + staerke() / 10 + groesse());

    n = rasse()->property("Handbonus").toInt();
    n = n + roll(1, 20);
    if                (n < 16) setHaendig(fl("rechtshndig"));
    else if (n > 15 && n < 20) setHaendig(fl("linkshndig"));
    else                       setHaendig(fl("beidhndig"));
    n = rasse()->property("Altersfaktor").toInt();
    setAlter(n * (roll(1,6) + 17));
}

int Figur::rollAP() {
    int newAP = roll(grad(), 3) + apBonus();
    return qMax(ausdauer(), newAP);
}

int Figur::ewStr(const QByteArray& prop, const Lernstoff* l) {
    QString sew = l->property(prop).toString();
    if (sew.toInt() != 0)
        return sew.toInt();
    Q_ASSERT(sew.length() > 2);
    foreach(QString stat, gStatNames.keys()) {
        if (sew.contains(stat)) {                   // Stat-Abkuerzung gefunden
            sew.replace(QRegExp(stat), property(gStatNames[stat].toLatin1()).toString());  // durch Zahl ersetzen
            QScriptEngine se;
            QScriptValue res = se.evaluate(sew);    // als Script auswerten
            if (!res.isError() && res.isNumber())
                return res.toInteger();
        }
    }
    return 0;
}

LernList Figur::getAngeboren() {
    LernList ret(true);
    foreach(const Lernstoff* l, Stoffe::l(ANGEBOREN)) {   // angeborenList aufbauen - nach "Min"-Attribut sortiert
        QStringList mins = l->property("Min").toString().split(',');
        QStringList ews  = l->property("Anfangswert").toString().split(',');
        Q_ASSERT(mins.length() == ews.length());
        for(int i = 0; i < mins.length(); i++) {    // Listen ausfalten
            int min = mins[i].toInt();
            Lernstoff* nl = l->clone();             // Lernstoff-Kopie
            nl->setProperty("Anfangswert", ews[i]); // sicherstellen dass es bei diesen Lernstoffen
            nl->setEW(ewStr("Anfangswert", nl));
            nl->setProperty("Min", min);            // nur einen Anf.-wert und Min gibt (keine Listen mehr)
            if (nl->name() == "Berserkergang" && EW(nl) < 1) {
                delete nl;
                continue;
            }
            int j;
            for(j = 0; j < ret.count(); j++)
                if (ret[j]->property("Min").toInt() > min)
                    break;
            ret.insert(j, nl);
        }
    }
    return ret;
}

void Figur::lerneTypisch() {
    foreach(Assoc a, klasse()->assocs.assocs(TYPISCH)) {
        QString ls = a.sWert;
        QStringList lsl = ls.split(',');
        foreach(Lernstoff* l, gelerntList)
            foreach(QString s, lsl)
                if (l->name().contains(s))
                    vergesse(l->name());
        if (lsl.count() > 1)
            ls = QInputDialog::getItem(NULL, "typische Eigenschaft", fl("Fertigkeit oder Zauber auswhlen:"), lsl);
        if (ls.length() < 2)            // Bei "Abbruch" nehmen wir halt die erste
            ls = lsl[0];
        lerne(ls)->setEW(a.iWert);
    }
}
    
void Figur::lerneAngeboren(const QString& l1, const QString& l2) {
    foreach(Lernstoff*l, getAngeboren())
        if (l->property("Min").toInt() < 1 && lernbar(l, LK_NONE))
            lerne(l);       // die ohne Min sind allgemeine angeborene Fertigkeiten
    lerne(l1);
    lerne(l2);
    foreach(QString ang, rasse()->angeboren) {
        int EW = ang.indexOf("+");
        Q_ASSERT(EW > 0 && EW < ang.length());
        QString lname = ang.left(EW);
        EW            = ang.right(ang.length() - EW).toInt();
        Lernstoff* l  = gelernt(lname);
        if (l != NULL) {                // todo - das hier fuehrt bei Gonmen die "Hoeren+2" wuerfeln zu Hoeren+4
            l->setEW(l->EW() + EW);     // wegen Zwergen-Raufen: der EW als Modifikator
            if (l->EW() == 0)
                vergesse(l->name());    // Wenn z.B. Richen-2 gewuerfelt und Riechen+2 als Rassenfertigkeit dann Riechen vergessen
        } else
            lerne(ang)->setEW(EW);
    }
}

void Figur::setSpezialisierung   (const QString& spezialisierung) {
    m_spezialisierung = spezialisierung;
    foreach(Lernstoff* w, gelerntList)
        if (isSpezialisierung(w))
            w->setEW(2);
}

AssocList Figur::fGruppenList() const {
    AssocList ret;
    foreach(Assoc a, klasse()->assocs)
        if (a.typ == GRUPPE)
            ret.addGruppe(a);
    if (ret[0].sWert != "Alltag") {
        Assoc a;
        a.typ = GRUPPE;
        a.sWert = "Alltag";
        a.iWert = 0;
        ret.insert(0, a);
    }
    QStringList standList = typGelernt(STAND)->property("ExtraLE").toString().split(',');       // todo - ",leicht" auswerten
    int i;
    for(i = 0; i < ret.count(); i++)
        if (ret[i].sWert == standList[0])
            break;
    if (i == ret.count()) { // die zum Stand gehoerige Gruppe gab's noch nicht, also neu anlegen
        Assoc a;
        a.typ = GRUPPE;
        a.sWert = standList[0];
        a.iWert = 0;
        ret.addGruppe(a);
        for(i = 0; i < ret.count(); i++)        // die neue Gruppe kann irgendwo gelandet sein
            if (ret[i].sWert == standList[0])   // deswegen nochmal danach suchen
                break;
    }
    Q_ASSERT(i < ret.count());
    ret[i].iWert += standList[1].toInt();       // jetzt die Standes-LE setzen
    return ret;
}

int Figur::ausdauerBonus() const {
    if (!alive())
        return 0;
    return konstitution() / 10 + staerke() / 20;
};

int Figur::apBonus() const {
    return ausdauerBonus() + grad() * klasse()->property("apbonus").toInt();
}

int Figur::schadenBonus() const {
    if (!alive())
        return 0;
    return staerke() / 20 + geschicklichkeit() / 30 - 3;
};

int Figur::angriffBonus() const {
    if (!alive())
        return 0;
    return statBonus(geschicklichkeit());
};

int Figur::abwehrBonus() const {
    if (!alive())
        return 0;
    return statBonus(gewandheit());
};

int Figur::zauberBonus() const {
    if (!alive())
        return 0;
    return statBonus(zaubertalent());
};

QString Figur::resistenzBonus() const {
    if (!alive())
        return "       /";
    int geist, koerper;
    geist = koerper = rasse()->property("Resistenz").toInt();
    if (geist == 0) {       // kein Rassenbonus, also stat-abhaengig
        geist   = statBonus(intelligenz());
        koerper = statBonus(konstitution());
    }
    if (klasse() != NULL)
        if (klasse()->isKAE())
            koerper++;
        else if (klasse()->isZAU()) {
            geist += 2;
            koerper += 2;
        }
    return itos(geist, false, true) + " / " + itos(koerper, false, true);
};

QString Figur::gestalt() const {
    QString ret = "";
    if (!alive())
        return ret;
    if      (groesse() < 166) ret = "klein / ";
    else if (groesse() > 180) ret = fl("gro / ");
    else                      ret = "mittel / ";
    int n = groesse() + rasse()->property("Norm").toInt();
    if      (gewicht() < n - n / 10) ret = ret + "schlank";
    else if (gewicht() > n + n / 10) ret = ret + "breit";
    else                             ret = ret + "normal";
    return ret;
};

QString Figur::itos(int i, bool zeroBlank, bool withPlus) const {

    if (!alive())
        return "";
    if (zeroBlank && (i == 0 || i < -5))
        return "";
    QString ret = i2s(i);
    if (withPlus && i > -1)
        ret = "+" + ret;
    return ret;
}

bool Figur::write(QString saveName) {
    EvaXmlDoc doc("figur");
    doc.root.setAttribute("name", name());
    QDomElement   de;
    for (int i = 0; i < metaObject()->propertyCount(); i++) {
        QMetaProperty prop = metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        de = doc.createElement(prop.name());
        de.setAttribute("Value", property(prop.name()).toString());
        doc.root.appendChild(de);
    }
    foreach (const Lernstoff* l, gelerntList) {
        de = doc.createElement("lernstoff");
        de.setAttribute("name", l->name());
        de.setAttribute("EW", l->EW());
        de.setAttribute("PP", l->PP());
        doc.root.appendChild(de);
        if (l->typ() == ZAUBER)     // Zauber-Beschreibungen kopieren, die koennten sich geaendert haben
            Stoffe::l(l->name())->setProperty("Beschreibung", l->property("Beschreibung").toString());
    }
    dinge.write(doc, doc.root);
    guthaben.write(doc, doc.root);
    // write the domDocument to the file
    if (saveName == "")
        if (name().length() > 1)   // Charaktername
            saveName = name() + itos(grad()) + ".eva";
        else
            saveName = "test.eva";
    doc.write(saveName);
    Stoffe::writeZauberDesc();
    dirty = false;
    return true;
} // Write

bool Figur::read(QString newName) {
    EvaXmlDoc doc("");
    if (!doc.read(newName) || doc.root.tagName() != "figur")
        return false;
    clear();
    for (int i = 0; i < doc.root.childNodes().size(); i++) {
        QDomElement e = doc.root.childNodes().at(i).toElement();
        if (e.nodeName() == "lernstoff") {
            Lernstoff* l = lerne(e.attribute("name"));
            l->setEW(e.attribute("EW").toInt());
            l->setPP(e.attribute("PP").toInt());
        }
        else if (e.nodeName() == "ding")
            dinge.read(e);
        else if (e.nodeName() == "guthaben")
            guthaben.read(e);
        else {
            QByteArray pn = e.nodeName().toLatin1();
            if (metaObject()->indexOfProperty(pn) < 0)
                continue;
            setProperty(pn, e.attribute("Value"));
        }
    }
    dirty = false;
    return true;
}

QString Figur::getFertigkeit(const Lernstoff* l, QString widgetName) const {
    if (l == NULL)
        return "";
    if (widgetName.contains("_pp"))
        return i2s(l->PP());
    QString n = widgetName.split('_')[1];   // die Nummer oder der Name
    QString plusEW = "";                    // das EW-Anhangsel
    if (!l->gelernt())
        plusEW = "+(" + i2s(l->EW()) + ")";
    else if (l->typ() == ZAUBER)
        plusEW = itos(EW(l) + EW("Zaubern"), false, true);
    else if (l->typ() == WAFFE) {
        plusEW = itos(EW(l) + gelernt(l->assocs.assocs(LERNPREREQ)[0].sWert)->EW(), false, true);   // Summe aus EW der Waffe und EW der Waffengruppe
    } else if (l->typ() == ZAUBERGRUPPE)
        plusEW = "";
    else
        plusEW = itos(EW(l), false, true);
    if (n.toInt() > 0 || n.contains(l->key()))
        if (l->typ() == ZAUBER || l->typ() == WAFFE)
            return l->name();
        else 
            return l->name() + plusEW;
    if (widgetName.contains("ew_"))       return plusEW;
    if (widgetName.contains("schaden_"))  return l->property("Schaden").toString();     // todo - schadensbonus() einbauen
    if (widgetName.contains("nah_"))      return l->property("Reichweite").toString();
    if (widgetName.contains("ap_"))       return l->property("AP").toString();
    if (widgetName.contains("dauer_"))    return l->property("Zauberdauer").toString();
    if (widgetName.contains("wb_"))       return l->property("Wirkungsbereich").toString();
    if (widgetName.contains("rw_"))       return l->property("Reichweite").toString();
    if (widgetName.contains("wd_"))       return l->property("Wirkungsdauer").toString();
    if (widgetName.contains("wirkung_"))  return Stoffe::l(l->name())->property("Beschreibung").toString(); // seltsam: die kurze Form tut manchmal nicht ??
    if (widgetName.contains("art_"))      return l->property("Ursprung").toString() + " " + l->property("Agens").toString() + "\n" + l->property("Prozess").toString() + " " + l->property("Reagens").toString();
    if (widgetName.contains("material_")) return "";
    return "";
}

QString Figur::widgetValue(const QString& widgetName) const {
/*
    Diese Funktion holt den Wert fuer ein Widget aus der Figur. Dabei gelten folgende Konventionen:

    <n> ist eine Zahl, die als Index in den zuvor angegebenen Lernstoff-Typ interpretiert wird.
    Es wird der n-te passende Lernstoff zurueckgegeben (beginnend bei 1).

    <property-name>     Wert der entsprechenden Property (siehe Header, z.B. staerke)
    <typ-name>          Bezeichner eines Lernstofftyps (gLernstoffTypNamen, nur unique), z.B. stand
    waffe_<n>           gelernte Waffengruppe
    zauber_<n>          gelernter Zauber
    angeboren_<n>       angeborene Fertigkeit (nicht steigerbar)
    ungelernt_<n>       ungelernte Fertigkeit
    fertigkeit_<n>      gelernte Fertigkeit
    fertigkeit_<name>   entsprechende Fertigkeit (z.B. fertigkeit_raufen = "Raufen +8")
    eptab_datum_<n>     Datum des n-ten Eintrags in die Guthaben-Liste
    eptab_ES_<n>        Erfahrungsschatz "
    eptab_EP_<n>        Erfahrungspunkte "
    eptab_geld_<n>      Geldguthaben (Gold) "
    glaube_label        "Glaube", "Gott" oder "Totem", je nach Glaube
    glaube              Name des Gottes oder Totems, oder "druidisch", oder "gleichgltig"
*/
    QStringList wnl = widgetName.split('_');
    while (wnl.count() < 3)
        wnl.append("");
    int index = wnl[1].toInt() - 1;
    if (index < 0)
        index = wnl[2].toInt() - 1;

    LernstoffTypen typ = FERTIGKEIT;
    if (widgetName.contains("waffe_"))
        typ = WAFFE;
    if (widgetName.contains("zauber_"))
        typ = ZAUBER;
    if (widgetName.contains("angeboren_"))
        typ = ANGEBOREN;
    LernList checkList(false);
    if (widgetName.contains("waffe_")) {
        QStringList dingNamen = dinge.dingNamen();
        foreach(Lernstoff* l, gelerntList) {
            if (l->typ() != WAFFE)
                continue;
            foreach(QString d, l->property("Ding").toString().split(','))   // alle Dinge die zu dieser Waffe passen (z.B. Wurfmesser passen zum Lernstoff Dolch)
                if (dingNamen.contains(d)) {
                    checkList.append(l);
                    break;                  // jede Waffe nur einmal aufnehmen, also abbrechen wenn wir eine gefunden haben
                }
        }
    } else if (widgetName.contains("ungelernt_")) {
        foreach(Lernstoff* l, Stoffe::l(FERTIGKEIT))
            if (l->property("EWungelernt").isValid() && (!alive() || lernbar(l, LK_NONE))) {
                l->setEW(l->property("EWungelernt").toInt());
                checkList.append(l);
            }
        foreach(Lernstoff* l, Stoffe::l(ANGEBOREN))     // Zaubern, Abwehr etc.
            if (l->property("EWungelernt").isValid()) {
                l->setEW(l->property("EWungelernt").toInt());
                checkList.append(l);
            }
        if (alive() && rasse()->property("ungelernt").isValid()) {
            QStringList ul = rasse()->property("ungelernt").toString().split(',');
            foreach(Lernstoff* l, checkList)
                foreach(QString u, ul)
                    if (u.contains(l->name()))
                        l->setEW(u.mid(u.indexOf("+")).toInt());
        }
        foreach(Lernstoff* l, checkList)
            if (gelernt(l->name()))
                checkList.removeAll(l);
        checkList.sort(LES_LAENDER);
    } else
        foreach(Lernstoff* l, gelerntList)
            if (  ( l->typ() == typ                         // bei passendem Typ nehmen wir auf
                    || (   typ == FERTIGKEIT                // ausserdem kommen Zauber- und
                        && (   l->typ() == ZAUBERGRUPPE     // Waffengruppen in die Liste mit
                            || l->typ() == WAFFENGRUPPE)    // den normalen Fertigkeiten
                       )
                  )
            && (typ != ANGEBOREN || l->name() != "AP verbessern") ) // schliesslich unterdruecken wir noch AP verbessern
            {
                checkList.append(l);
            }
    Lernstoff* l = NULL;
    if (index >= 0 && index < checkList.count())
        l = checkList[index];
    if (index < 0)
        l = gelernt(wnl[1]);
    if (l == NULL && !alive() && index < 0) {
        l = Stoffe::l(wnl[1]);
        if (l != NULL)
            return l->name() + " +   ";     // angeborene Fertigkeit auf leeren Boegen schreiben (z.B. "Raufen + ")
        else
            return "";
    }
    // l zeigt jetzt auf den Lernstoff, der zum Widget passt
        if  (widgetName.contains("eptab"))          return guthaben.widgetValue(widgetName);
    else if (widgetName.contains("item"))           return dinge.widgetValue(widgetName);
    else if (widgetName.contains("magic_"))         return dinge.widgetValue(widgetName);
    else if (widgetName.contains("fertigkeit_")
         ||  widgetName.contains("waffe_")
         ||  widgetName.contains("zauber_")
         ||  widgetName.contains("angeboren_")
         ||  widgetName.contains("ungelernt_"))     return getFertigkeit(l, widgetName);
    else if (widgetName.contains("glaube")) {
        if (widgetName == "glaube_label") {
            if (glaube() == fl("Gtter"))               return "Gott";
            if (gelernt(TOTEM).length() == 1)       return "Totem";
            return "Glaube";
        }
        LernList ll(false);
        if (glaube() == fl("Gtter"))
            ll = gelernt(GOTT);
        if (glaube() == "schamanistisch")
            ll = gelernt(TOTEM);
        if (ll.length() == 1)
            return ll[0]->name();
        return glaube();
    } else { // passende Property suchen 
        QByteArray pn = wnl[0].toLatin1();
        if (metaObject()->indexOfProperty(pn) > -1)
            if (wnl[0].contains("Bonus") && !wnl[0].contains("esistenz"))
                return itos(property(pn).toInt(), false, true);
            else
                return property(pn).toString();
    }
    // Fuer die Hintergrundinfo: pruefen ob das Label dem Namen eines Lernstofftyps entspricht
    QString wText = "";
    if (typGelernt(widgetName) != NULL) {
        wText = typGelernt(widgetName)->name();
        l = Stoffe::l(wText);
        if (l->typ() == RASSE || l->typ() == KLASSE)
            wText = ((Rasse*)l)->name_mw(frau());
    }
    return wText;
}

void Figur::setWidgetValue(const QByteArray& name, const QVariant& value) {
    if (metaObject()->indexOfProperty(name) > - 1)
        // fieldName entspricht einer Property -> einfach setzen
        setProperty(name, value);
    else
        // keine Property -> ist der Inhalt vielleicht ein Lernstoff?
        if (Stoffe::keys().contains(lernKey(value.toString())))
            lerne(value.toString());
}

void Figur::addEPgeld(int EP, int gold) {
    Guthaben g = guthaben.val();
    g.setES(g.ES() + EP);
    g.setEP(g.EP() + EP);
    g.setGold(g.gold() + gold);
    while (g.ES() >= klasse()->gradEP(grad())) {
        setGrad(grad() + 1);
        DlgGradNeu  gn(this);
        gn.exec();
        int apbonus = klasse()->property("apbonus").toInt();
        if (grad() > 5)
            apbonus = 0;
        setAusdauer(ausdauer() + gn.ui.ap->text().toInt() + apbonus);
        QString eigenschaft = lowerUmlaut(gn.ui.eigenschaft->text());
        if (shortStatName(eigenschaft) != "") {
            int e = property(eigenschaft.toLatin1()).toInt();
            e = qMin(100, e + gn.ui.w3p1->text().toInt());
            setProperty(eigenschaft.toLatin1(), e);
        }
    }
    guthaben.append(g);
}
