#pragma once

#include "lernstoff.h"
#include "ding.h"

#define qMinMax(min,max,val)   qMax(min, qMin(max, val))
// stellt sicher dass min <= val <= max

class Figur : public QObject
{
    Q_OBJECT

public:
    Figur(QString fname = "");
    Figur(Figur* f);
    ~Figur();

    void        clear();
    bool        alive() const;                              // Name nicht leer und staerke != 0

// Funktionen fuer Lernstoffe
    Lernstoff*  lerne(const QString& name);                 // ohne Kosten lernen (z.B. bei Erschaffung)
    Lernstoff*  lerne(Lernstoff* l);                        // wie oben, nur mit Lernstoff* als Parameter
    void        lerne(Lernstoff* l, int ep, int geld, int pp);// fuer EP etc. lernen
    void        vergesse(const QString name);               // gelernten Stoff wieder vergessen
    LernList    gelernt(LernstoffTypen typ) const;          // alle gelernten von typ
    Lernstoff*  gelernt(const QString name) const;          // gelernten Stoff zurueckgeben, NULL wenn unbekannt

    bool        lernbar(Lernstoff* l, eLernkostentyp kosten) const; // prueft Einschraenkungen durch Klasse, Stats etc., kosten/guthaben auswerten
    LernList    lernbar(LernstoffTypen typ, eLernkostentyp kosten) const;  // Liste von Kopien von lernbaren Stoffen
    QStringList zusaetze(Lernstoff* l) const;               // Liste der lernbaren Zusaetze
    const Lernstoff* typGelernt(LernstoffTypen typ) const;  // Der hchstens eine Stoffes von typ, den wir gelernt haben, NULL sonst
    const Lernstoff* typGelernt(const QString typ) const;   // wie oben, nur typ als String
    Rasse*      rasse() const;                              // wie oben, gibt Rasse bzw. NULL zurueck
    Klasse*     klasse() const;                             // selbes fuer Klasse
    void        setKosten(Lernstoff* l, eLernkostentyp kosten) const; // setzt LE- bzw. TE-Kosten (abhaengig von l->gelernt())
    int         EPKosten(const Lernstoff* l, int pp) const; // EP um diesen Lernstoff zu lernen/verbessern (pp sind die ausgegebenen PP, -1 fuer max)
    int         geldKosten(const Lernstoff* l, int pp) const; // Mindest-Geld, um diesen Stoff zu lernen/verbessern
    QStringList muttersprachen() const;
    QStringList zweitsprachen() const;
    bool        isSpezialisierung(const Lernstoff* l) const;    // true wenn l spezialisiert ist

    int         EW(const QString name) const;               // durch Leiteigenschaft modifizierter
    int         EW(const Lernstoff* l) const;               // EW der Fertigkeit
    int         PP(const Lernstoff* l) const;               // fuer diesen Stoff ausgebbare PP
    QString     EWStr(const Lernstoff* l) const;            // EW als String (mit Vorzeichen)
    int         ewStr(const QByteArray& prop, const Lernstoff* l); // EW aus l->property(prop) holen (ggf. Skript interpretieren)

    bool        read(const QString name);                   // Figur aus XML-Datei name lesen (defautl-Extension ".eva")
    bool        write(const QString name = "");             // Figur in die XML-Datei name schreiben, "" = "test.eva"

// Schnittstelle zur Bedienoberflaeche
    void        rollBaseStats();                            // Die sechs Basis-Eigenschaften auswuerfeln
    void        rollDerivedStats();                         // alle uebrigen Eigenschaften auswuerfeln
    int         rollAP();                                   // aktuelle AP auswuerfeln

    QString     widgetValue(const QString& widgetName) const; // liefert den Wert zu einem Widget-Namen
    void        setWidgetValue(const QByteArray& name, const QVariant& value); // name ist ein Widget-Name, value der Wert, Figur wird gesetzt
    QString     itos(int i, bool zeroBlank = false, bool withPlus = false) const;
    AssocList   fGruppenList() const;                       // Liste mit Fertigkeitsgruppen-Namen (als LE_KOSTEN - Assocs)

    void        lerneTypisch();                             // die typischen Fertigkeiten lernen
    LernList    getAngeboren();                             // Liste mit allen moglichen angeborenen Eigenschaften (automatische zuerst, dann wuerfelbare)
    Lernstoff*  lernbar(const QString& name) const;         // Der lernbare Stoff
    void        lerneAngeboren(const QString& l1, const QString& l2); // Eintraege von getAngeboren (bei 100 gibt's zwei)

    void        addEPgeld(int EP, int geld);                // EP und Geld einnehmen, neues Guthaben anhaengen (ES / Grad pruefen)

// globale Variablen
    bool        dirty;                                      // Ungesicherte Aenderungen?
    LernList    gelerntList;                          // Liste aller gelernten Stoffe (Kopien, nicht einfach Zeiger in Stoffe::liste())
    DingList    dinge;                                      // Gegenstaende-Liste
    GuthabenList guthaben;                                  // Geld, EP und ES mit Datum

private:
    int         rollStat(const QString statName, bool zweimal = true, int numdice = 1, int type = 100);  // beruecksichtigt rassenspez. Eigenarten
    QString     getFertigkeit(const Lernstoff* l, QString name) const;  // Hilfsfunktion fuer widgetValue

// ab hier gibt's nur noch Properties und ihre Verwaltung.
private:
    Q_PROPERTY(QString name                 READ name WRITE setName)
    Q_PROPERTY(QString spieler              READ spieler WRITE setSpieler)
    Q_PROPERTY(bool    frau                 READ frau WRITE setFrau)
    Q_PROPERTY(int     grad                 READ grad WRITE setGrad)
    Q_PROPERTY(QString gastland             READ gastland WRITE setGastland)
    Q_PROPERTY(QString spezialisierung      READ spezialisierung WRITE setSpezialisierung)
    Q_PROPERTY(int     staerke              READ staerke WRITE setStaerke)
    Q_PROPERTY(int     geschicklichkeit     READ geschicklichkeit WRITE setGeschicklichkeit)
    Q_PROPERTY(int     gewandheit           READ gewandheit WRITE setGewandheit)
    Q_PROPERTY(int     konstitution         READ konstitution WRITE setKonstitution)
    Q_PROPERTY(int     intelligenz          READ intelligenz WRITE setIntelligenz)
    Q_PROPERTY(int     zaubertalent         READ zaubertalent WRITE setZaubertalent)
    Q_PROPERTY(int     aussehen             READ aussehen WRITE setAussehen)
    Q_PROPERTY(int     ausstrahlung         READ ausstrahlung WRITE setAusstrahlung)
    Q_PROPERTY(int     bewegungsweite       READ bewegungsweite WRITE setBewegungsweite)
    Q_PROPERTY(int     willenskraft         READ willenskraft WRITE setWillenskraft)
    Q_PROPERTY(int     ausdauerBonus        READ ausdauerBonus)
    Q_PROPERTY(int     schadenBonus         READ schadenBonus)
    Q_PROPERTY(int     angriffBonus         READ angriffBonus)
    Q_PROPERTY(int     abwehrBonus          READ abwehrBonus)
    Q_PROPERTY(int     zauberBonus          READ zauberBonus)
    Q_PROPERTY(QString resistenzBonus       READ resistenzBonus)
    Q_PROPERTY(int     leben                READ leben WRITE setLeben)
    Q_PROPERTY(int     ausdauer             READ ausdauer WRITE setAusdauer)
    Q_PROPERTY(int     gnade                READ gnade WRITE setGnade)
    Q_PROPERTY(int     gunst                READ gunst WRITE setGunst)
    Q_PROPERTY(QString glaube               READ glaube WRITE setGlaube)
    Q_PROPERTY(int     alter                READ alter WRITE setAlter)
    Q_PROPERTY(QString geburtsdatum         READ geburtsdatum WRITE setGeburtsdatum)
    Q_PROPERTY(QString gestalt              READ gestalt)
    Q_PROPERTY(int     gewicht              READ gewicht WRITE setGewicht)
    Q_PROPERTY(int     groesse              READ groesse WRITE setGroesse)
    Q_PROPERTY(QString haendig              READ haendig WRITE setHaendig)
    Q_PROPERTY(QString merkmale             READ merkmale WRITE setMerkmale)
    
public:
    QString name()                  const {return m_name;};
    QString spieler()               const {return m_spieler;};
    bool    frau()                  const {return m_frau;};
    int     grad()                  const {return m_grad;};
    QString gastland()              const {return m_gastland;};
    QString spezialisierung()       const {return m_spezialisierung;};
    int     staerke()               const {return m_staerke;};
    int     geschicklichkeit()      const {return m_geschicklichkeit;};
    int     gewandheit()            const {return m_gewandheit;};
    int     konstitution()          const {return m_konstitution;};
    int     intelligenz()           const {return m_intelligenz;};
    int     zaubertalent()          const {return m_zaubertalent;};
    int     aussehen()              const {return m_aussehen;};
    int     ausstrahlung()          const {return m_ausstrahlung;};
    int     willenskraft()          const {return m_willenskraft;};
    int     bewegungsweite()        const {return m_bewegungsweite;};
    int     ausdauerBonus()         const;      // AP-Bonus der sich aus der Konstitution ergibt
    int     apBonus()               const;      // effektiver AP-Bonus (das + hinter den Wuerfeln, incl. Grad)
    int     schadenBonus()          const;
    int     angriffBonus()          const;
    int     abwehrBonus()           const;
    int     zauberBonus()           const;
    QString resistenzBonus()        const;
    int     leben()                 const {return m_leben;};
    int     ausdauer()              const {return m_ausdauer;};
    int     gnade()                 const {return m_gnade;};
    int     gunst()                 const {return m_gunst;};
    QString glaube()                const {return m_glaube;};
    int     alter()                 const {return m_alter;};
    QString geburtsdatum()          const {return m_geburtsdatum;};
    QString gestalt()               const;
    int     gewicht()               const {return m_gewicht;};
    int     groesse()               const {return m_groesse;};
    QString haendig()               const {return m_haendig;};
    QString merkmale()              const {return m_merkmale;};
//p1

    void setName              (const QString& name)         {m_name = name;};
    void setSpieler           (const QString& spieler)      {m_spieler = spieler;};
    void setFrau              (bool  frau)                  {m_frau = frau;};
    void setGrad              (int   grad)                  {m_grad = grad;};
    void setGastland          (const QString& gastland)     {m_gastland = gastland;};
    void setSpezialisierung   (const QString& spezialisierung);
    void setStaerke           (int   staerke)               {m_staerke = qMinMax(1, 100, staerke);};
    void setGeschicklichkeit  (int   geschicklichkeit)      {m_geschicklichkeit = qMinMax(1, 100, geschicklichkeit);};
    void setGewandheit        (int   gewandheit)            {m_gewandheit = qMinMax(1, 100, gewandheit);};
    void setKonstitution      (int   konstitution)          {m_konstitution = qMinMax(1, 100, konstitution);};
    void setIntelligenz       (int   intelligenz)           {m_intelligenz = qMinMax(1, 100, intelligenz);};
    void setZaubertalent      (int   zaubertalent)          {m_zaubertalent = qMinMax(1, 100, zaubertalent);};
    void setAussehen          (int   aussehen)              {m_aussehen = qMinMax(1, 100, aussehen);};
    void setAusstrahlung      (int   ausstrahlung)          {m_ausstrahlung = qMinMax(1, 100, ausstrahlung);};
    void setBewegungsweite    (int   bewegungsweite)        {m_bewegungsweite = bewegungsweite;};
    void setWillenskraft      (int   willenskraft)          {m_willenskraft = qMinMax(1, 100, willenskraft);};
    void setLeben             (int   leben)                 {m_leben = leben;};
    void setAusdauer          (int   ausdauer)              {m_ausdauer = ausdauer;};
    void setGnade             (int   gnade)                 {m_gnade = gnade;};
    void setGunst             (int   gunst)                 {m_gunst = gunst;};
    void setGlaube            (const QString& glaube)       {m_glaube = glaube;};
    void setAlter             (int   alter)                 {m_alter = alter;};
    void setGeburtsdatum      (const QString& geburtsdatum) {m_geburtsdatum = geburtsdatum;};
    void setGewicht           (int   gewicht)               {m_gewicht = gewicht;};
    void setGroesse           (int   groesse)               {m_groesse = groesse;};
    void setHaendig           (const QString& haendig)      {m_haendig = haendig;};
    void setMerkmale          (const QString& merkmale)     {m_merkmale = merkmale;};
//p2

private:
    QString m_name;
    QString m_spieler;
    bool    m_frau;
    int     m_grad;
    QString m_gastland;
    QString m_spezialisierung;
    int     m_staerke;
    int     m_geschicklichkeit;
    int     m_gewandheit;
    int     m_konstitution;
    int     m_intelligenz;
    int     m_zaubertalent;
    int     m_aussehen;
    int     m_ausstrahlung;
    int     m_bewegungsweite;
    int     m_willenskraft;
    int     m_leben;
    int     m_ausdauer;
    int     m_gnade;
    int     m_gunst;
    QString m_glaube;
    int     m_alter;
    QString m_geburtsdatum;
    int     m_gewicht;
    int     m_groesse;
    QString m_haendig;
    QString m_merkmale;
//p3
};  // class Figur

// Ein paar Hilfsfunktionen
int             roll (int n, int w);
