#include "lernstoff.h"

#include <qdiriterator.h>
#include <qdebug.h>
#include <qmessagebox.h>
#include <qsettings.h>

// Liste von statischen Attributen (dynamic properties), die wir 1:1 uebernehmen
QStringList propertyAttributes = QStringList() 
    << "EWungelernt" << "Kategorie" << "Attribut" << "Stufe" << "Ding" << "Heimisch"
    << "Schaden" << "Min" << "Fertigkeit" << "Verkehrssprache" << "Spezial" << "ExtraLE"
    << "AP" << "Agens" << "Prozess" << "Spruchrolle" << "Reagens" << "Reichweite" << "Art" << "Ursprung" 
    << "Wirkungsbereich" << "Wirkungsdauer" << "Wirkungsziel" << "Zauberdauer";

QStringList statLimits = QStringList()
    << "MinSt" << "MinGs" << "MinGw" << "MinKo" << "MinIn" << "MinZt" << "MinpA"
    ;

QString gLernstoffTypNamen[LAST_LERNSTOFF] = {
    "Gott", "Totem", "Land", "Stand", "Rasse", "Klasse",
    "angeboreneFertigkeit", "Zaubergruppe", "Waffengruppe", "Fertigkeit", "Waffe", "Zauber"
};

QStringList fertigkeitsGruppen = QStringList() 
    << "Alltag" << "Freiland" << "Halbwelt" << "Kampf" << fl("Krper") 
    << "Sozial" << "Unterwelt" << "Waffen" << "Wissen" << "Zauber";

QHash<QString, QString> gStatNames;  // gStatNames[Abk] = langer Name

//***************** Lernstoff

Lernstoff::Lernstoff() : QObject() {
    m_name = "";
    m_key = "";
    m_typ = LAST_LERNSTOFF;
    m_gelernt = false;
    m_EW = 0;
    m_pp = 0;
    m_kosten = UNLERNBAR;
    m_zusaetze.clear();
    m_zusatz = -1;
    assocs.clear();
}

Lernstoff::~Lernstoff() {
    QString s = name();
    Lernstoff *l = Stoffe::l(s);
    if (l == this)
        int i = 0;
}

void Lernstoff::setDynProp(const QString& name, const QVariant& val) {
    if (val.isNull())
        return;
    setProperty(name.toLatin1(), val);
}

void Lernstoff::setPropAttr(const QString& propName, const QString& attrName, const QString& nodeName) {
    QVariant val;
    if (attrName != "" || nodeName != "")
        val = oldatt(attrName, nodeName);
    else
        val = oldatt(propName);
    setDynProp(propName, val);
}

Lernstoff::Lernstoff(const QDomElement actNode) {
    m_node         = actNode;
    m_gelernt    = false;
    m_EW         = -1;
    m_pp         = 0;
    m_kosten     = UNLERNBAR;
    m_zusatz     = -1;
    QString tn   = m_node.tagName();
    for (m_typ = GOTT; m_typ < LAST_LERNSTOFF; m_typ = LernstoffTypen(m_typ + 1))
        if (gLernstoffTypNamen[m_typ] == tn)
            break;
    setName(m_node.attribute("Name"));
    Q_ASSERT(m_name.length() > 1);
    if (m_typ == LAST_LERNSTOFF) {
        if (tn == "Spezies")        m_typ = RASSE;
        if (tn == "Typ")            m_typ = KLASSE;
        if (tn == "Spruch")         m_typ = ZAUBER;
        if (tn == "Sprache")        m_typ = FERTIGKEIT;
        if (tn == "Waffen-Grundkenntnis") m_typ = WAFFENGRUPPE;
    }
    Q_ASSERT(m_typ != LAST_LERNSTOFF);
    if (typ() == ANGEBOREN)
        setPropAttr("Anfangswert");
    foreach(QString prop, propertyAttributes) {
        if (prop == "Schaden") {
            QString s = oldatt("Schaden");
            QString p = oldatt("Schadensbonus");
            if (p != "" && !p.contains("+") && !p.contains("-"))
                s = s + "+";
            s = s + p;
            setDynProp(prop, s);
        } else
            setPropAttr(prop);
    }
    foreach(QString sm, statLimits) {
        int min = oldatt(sm).toInt();
        QString stat = sm.right(2);
        if (stat.length() < 2 || min < 10)
            continue;
        Assoc a;
        a.typ = STATPREREQ;
        a.sWert = gStatNames[stat];
        a.iWert = min;
        assocs.append(a);
    }
    for(QDomElement n = m_node.firstChildElement(); !n.isNull(); n = n.nextSiblingElement()) {
        // Alle Tags pruefen, zuerst diejenigen welche Assocs sind
        QString tag = n.tagName();
        Assoc a;
        a.iWert = -1;
        a.sWert = "";
        a.typ = LAST_ASSOC;
        if (tag.contains("Voraussetzungen")) {
            for (int i = 0; i < n.attributes().length(); i++) {
                QDomAttr attr = n.attributes().item(i).toAttr();
                if (gStatNames.keys().contains(attr.name())) {
                    a.typ = STATPREREQ;
                    a.sWert = gStatNames[attr.name()];
                    a.iWert = attr.value().toInt();
                    assocs.append(a);
                    a.typ = LAST_ASSOC;
                }
                if (attr.name() == "kannZaubern") {
                    a.typ = GROUPPREREQ;
                    a.sWert = "ZAU";
                }
                if (a.typ < LAST_ASSOC) 
                    assocs.append(a);
                a.typ = LAST_ASSOC;
            }   // for attributes
        } // </Voraussetzungen>
        if (fertigkeitsGruppen.contains(tag)) {
            a.typ = GRUPPE;
            a.sWert = tag;
            a.iWert = n.attribute("LE").toInt();
        }
        if (tag == "Typisch") {
            a.typ = TYPISCH;
            a.sWert = n.attribute("Name");
            a.iWert = n.attribute("EW").toInt();
        }
        if (tag == "Land") {        // <Land> in <Spezies> oder <Gott> (<Land> in <Kontinent> wird vom Parser abgefangen)
            a.typ = ALLOWED;
            a.sWert = n.attribute("Name");
            a.iWert = LAND;
        }
        if (tag == "Sprache" && (typ() == RASSE || typ() == LAND))
            setProperty("Sprache", n.attribute("Name"));
        if (tag == "Nachbarland") {
            QString s = "";
            if (property("Nachbarland").isValid())
                s = property("Nachbarland").toString() + ",";
            s = s + n.attribute("Name");
            setProperty("Nachbarland", s);
        }
        if (tag == "Typ") {        // <Typ>   in <Spezies> oder <Gott> (<Typ>  in <Typen>     wird vom Parser abgefangen)
            a.typ = ALLOWED;
            a.sWert = n.attribute("Name");
            a.iWert = KLASSE;
            if (a.sWert[0] == '-') {
                a.sWert.remove(0, 1);
                a.typ = FORBIDDEN;
            }
        }
        if (tag == "Spezies") {     // <Spezies> in <Gott>
            a.typ = ALLOWED;
            a.sWert = n.attribute("Name");
            a.iWert = RASSE;
        }
        if (tag == "Herkunft") {    // <Herkunft> in <Typ>
            a.typ = ALLOWED;
            a.sWert = n.attribute("Land");
            a.iWert = LAND;
        }
        if (tag == "angeboreneFertigkeit") {    // <angeboreneFertigkeit in <Typ>
            if (n.attribute("Art") == "v") {
                a.typ = FORBIDDEN;
                a.sWert = n.attribute("Name");
            }
            if (n.attribute("Art") == "f") {
                QString s = "";
                if (property("ungelernt").isValid())
                    s = property("ungelernt").toString() + ",";
                s = s + n.attribute("Name") + "+" + n.attribute("Erfolgswert");
                setProperty("ungelernt", s);
            }
        }
        if (a.typ < LAST_ASSOC)
            assocs.append(a);
        // Jetzt die Tags welche keine Assoc sind
        if (tag == "Besitz")
            setProperty("Besitz", oldatt("Name", "Besitz"));
        if (tag == "Beschreibung")
            setPropAttr("Beschreibung", "", "Beschreibung");
        if (tag == fl("Zustze"))
            m_zusaetze.append(n.attribute("Name"));
        if (tag == "Verkehrssprache")
            setProperty("verkehrssprache", oldatt("Verkehrssprache"));
        if (tag == "Reichweite" && n.attribute("nah").length() > 1)
            setProperty("Reichweite", n.attribute("nah") + "/" + n.attribute("mittel") + "/" + n.attribute("fern"));
    } // for n = ChildElement
    if (m_node.attribute("Grundkenntnisse") != "") {
        Assoc a;
        a.typ = LERNPREREQ;
        a.sWert = m_node.attribute("Grundkenntnisse");
        a.iWert = -1;
        assocs.append(a);
    }
    if (m_node.attribute("Erfolgswert") != "")
        setEW(m_node.attribute("Erfolgswert").toInt());
}

Lernstoff::Lernstoff(const Lernstoff *l) {
    Q_ASSERT(l != NULL);
    for(int i = 0; i < l->metaObject()->propertyCount(); i++) {
        QMetaProperty prop = l->metaObject()->property(i);
        if (!prop.isWritable())
            continue;
        QByteArray name = prop.name();
        setProperty(name, l->property(name));
    }
    foreach(QByteArray pn, l->dynamicPropertyNames())
        setProperty(pn, l->property(pn));
    m_node       = l->m_node;
    m_zusaetze   = l->m_zusaetze;
    foreach(Assoc a, l->assocs)
        assocs.append(a);
    memcpy(TEkosten, l->TEkosten, sizeof(int)*20);
}

Lernstoff* Lernstoff::clone() const {
    return new Lernstoff (this);
}

bool Lernstoff::operator==(Lernstoff const& rhs) const {
    if (this == NULL) 
        return false;
    if (&rhs == NULL)
        return false;
    return *this == rhs.name() && zusatz() == rhs.zusatz();
}

bool Lernstoff::operator==(QString const& rhs) const {
    if (this == NULL) 
        return false;
    if (&rhs == NULL)
        return false;
    return (name() == rhs || key() == rhs);
}

bool Lernstoff::operator!=(Lernstoff const& rhs) const {
    if (this == NULL) 
        return true;
    if (&rhs == NULL)
        return true;
    return *this != rhs.name() || zusatz() != rhs.zusatz();
}

bool Lernstoff::operator!=(QString const& rhs) const {
    if (this == NULL) 
        return true;
    if (&rhs == NULL)
        return true;
    return (name() != rhs && key() != rhs);
}

QString Lernstoff::name() const {
    QString ret = m_name;
    if (zusatz() >= 0)
        ret.append(":" + zusaetze().at(zusatz()));
    return ret;
}

void Lernstoff::setName(const QString& name) {
    if (name.contains(':'))
        m_name = name.left(name.indexOf(':'));
    else
        m_name = name;
    m_key  = lernKey(name); 
};

QString Lernstoff::oldatt(const QString attrName, const QString n1, const QString n2) const {
    QDomElement n = m_node;
    if (n1 != "" && !n.isNull())
        n = n.firstChildElement(n1);
    if (n2 != "" && !n.isNull())
        n = n.firstChildElement(n2);
    if (n.isNull())
        return "";
    if (attrName == "")
        return n.text();
    else
        return n.attribute(attrName);
}

QStringList Lernstoff::zusaetze() const {
    return m_zusaetze;
}

bool Lernstoff::isGruppe(const QString& gruppe) const {
    QStringList gruppen;
    foreach(Assoc a, assocs) {
        if (a.typ == LE_KOSTEN)
            gruppen.append(a.sWert);
        if (typ() == ZAUBER && a.typ == LERNPREREQ)
            gruppen.append(a.sWert);
    }
    foreach(QString g, gruppe.split(','))
        if (gruppen.contains(g))
            return true;
    return false;
}

int Lernstoff::rollProp(const QString& prop) const {
    int n, w, p;
    QByteArray s = property(prop.toLatin1()).toString().toLatin1();
    int ret = sscanf(s, "%iW%i%i", &n, &w, &p);
    if (ret == 2)
        p = 0;
    Q_ASSERT(ret == 3 || ret == 2);
    return roll(n, w) + p;
}

QStringList Lernstoff::landesSprachen(bool nachbarn) const {
    QStringList sprachen;

    if (typ() != LAND)
        return sprachen;
    if (!nachbarn) {
        if (property("Sprache").isValid())
            sprachen.append(property("Sprache").toString().split(','));
    } else {
        if (property("Nachbarn").isValid())
            foreach(QString n, property("Nachbarland").toString().split(','))
            sprachen.append(Stoffe::l(n)->landesSprachen());
        if (property("Verkehrssprache").isValid())
            foreach(QString v, property("Verkehrssprache").toString().split(','))
            sprachen.append(v);
    }
    sprachen.removeDuplicates();    // falls zwei Nachbarlaender dieselbe Sprache haben
    return sprachen;
}

void Lernstoff::setKosten(eLernkostentyp kosten) {
    m_lernkostentyp = kosten;
    if (kosten == LK_NONE || typ() != FERTIGKEIT && typ() != WAFFENGRUPPE && typ() != ZAUBER)
        setKosten(0);
    else if (gelernt())
        setKosten(TEkosten[EW()]);
    else
        foreach(Assoc a, assocs)
        if (typ() != ZAUBER && a.typ == LE_KOSTEN)
            setKosten(a.iWert);
        else if (typ() == ZAUBER && a.typ == LERNPREREQ)
            setKosten(a.iWert);
}

QDomElement Lernstoff::write(EvaXmlDoc& doc, QDomElement& parent) const {
    QDomElement le = doc.createElement("Lernstoff");
    le.setAttribute("Name", name());
    le.setAttribute("Typ", gLernstoffTypNamen[typ()]);
    if (EW() != -1)
        le.setAttribute("EW", EW());
    if (PP() != 0)
        le.setAttribute("PP", PP());
    if (m_zusaetze.size() > 0) {
        QDomElement ze = doc.createElement("Zusaetze");
        ze.setAttribute("Zus", m_zusaetze.join(","));
        le.appendChild(ze);
    }
    QList<QByteArray> dpl = dynamicPropertyNames();
    if (dpl.count() > 0) {
        QDomElement dpe = doc.createElement("Dynprops");
        QDomElement be;
        foreach(QByteArray dynProp, dpl)
            if (dynProp.contains("Beschreibung")) {
                // Beschreibungstexte weglassen aus lizenzrechtlichen Gruenden
                /*
                be  = doc.createElement("Beschreibung");
                be.setAttribute("Text", property(dynProp).toString());
                */
            } else if (property(dynProp).toString() != "")
                dpe.setAttribute(dynProp, property(dynProp).toString());
            le.appendChild(dpe);
            if (!be.isNull())
                le.appendChild(be);
    }
    foreach(Assoc a, assocs)
        a.write(doc, le);
    if (typ() == WAFFENGRUPPE || typ() == FERTIGKEIT) {
        QDomElement tke = doc.createElement("TEKosten");
        for(int i = 0; i < 20; i++)
            tke.setAttribute("G" + i2s(i), i2s(TEkosten[i]));
        le.appendChild(tke);
    }
    parent.appendChild(le);
    return le;
}

void Lernstoff::read(const QDomElement& de) {
    setEW(-1);
    for(QDomNode n = de.firstChild(); !n.isNull(); n = n.nextSibling())
        if (n.isElement()) {
            QDomElement ne = n.toElement();
            if (n.nodeName() == "Assoc") {
                Assoc a;
                a.read(ne);
                assocs.append(a);
            }
            if (n.nodeName() == "Zusaetze")
                m_zusaetze = ne.attribute("Zus").split(',');
            if (n.nodeName() == "Beschreibung")
                setProperty("Beschreibung", ne.attribute("Text"));
            if (n.nodeName() == "TEKosten")
                for(int i = 0; i < 20; i++)
                    TEkosten[i] = ne.attribute("G" + i2s(i)).toInt();
            if (n.nodeName() == "Dynprops")
                for(int i = 0; i < ne.attributes().count(); i++)
                    setProperty(ne.attributes().item(i).nodeName().toLatin1(), ne.attributes().item(i).nodeValue());
        }
        for(int i = 0; i < de.attributes().count(); i++) {
            QString nn = de.attributes().item(i).nodeName();
            QString nv = de.attributes().item(i).nodeValue();
            if (nn == "Name")
                setName(nv);
            else if (nn == "Typ")
                setTyp((LernstoffTypen)findName(nv, gLernstoffTypNamen, LAST_LERNSTOFF));
            else if (nn == "EW")
                setEW(nv.toInt());
            else if (nn == "PP")
                setPP(nv.toInt());
            else
                setProperty(nn.toLatin1(), nv);
        }
        if (typ() == ZAUBER || typ() == WAFFE)
            setEW(0);
}

//***************** Rasse

Rasse::Rasse(const QDomElement node) : Lernstoff(node) {
    if (node.tagName() == "Spezies") {
        setPropAttr("Name-m", "Name-maennlich");
        setPropAttr("Name-w", "Name-weiblich");
    }
    if (node.tagName() == "Typ") {
        setPropAttr("Name-m", "Bezeichnung-Mann");
        setPropAttr("Name-w", "Bezeichnung-Frau");
    }
    for(QDomElement n = m_node.firstChildElement("angeboreneFertigkeit"); !n.isNull(); n = n.nextSiblingElement("angeboreneFertigkeit")) {
        if (n.attribute("Art") == "a")
            angeboren.append(n.attribute("Name"));
        if (n.attribute("Art") == "e")
            erschaffungErlaubt.append(n.attribute("Name"));

    }

    setPropAttr("Altersfaktor", "val", "Altersfaktor");
    setPropAttr("Groesse", "val", "Groesse");
    setPropAttr("Gewicht", "val", "Gewicht");
    setPropAttr("Norm", "val", "Norm");
    setPropAttr("Bewegungsweite", "val", "Bewegungsweite");
    setPropAttr("Handbonus");
    setPropAttr("LPbonus", "LP_Bonus", "Modifikation");
    setPropAttr("Resistenz", "Resistenz", "Modifikation");
    if (oldatt("Min", "Modifikation", "Stand").toInt() != 0)
        setDynProp("StandMin", oldatt("Min", "Modifikation", "Stand"));
    if (oldatt("Max", "Modifikation", "Stand").toInt() != 0)
        setDynProp("StandMax", oldatt("Max", "Modifikation", "Stand"));
    if (oldatt("extra", "Modifikation", "LEKosten").toInt() != 0)
        setDynProp("LEKosten", oldatt("extra", "Modifikation", "LEKosten"));
    foreach(QString stat, gStatNames.keys()) {
        int val = oldatt(stat, "Modifikation", "Grundwerte").toInt();
        if (val != 0)
            setProperty(stat.toLatin1(), val);
    }
};

Rasse::Rasse(const Rasse* r) : Lernstoff(r) {
    Q_ASSERT(r->typ() == RASSE || r->typ() == KLASSE);
    angeboren  = r->angeboren;
    erschaffungErlaubt = r->erschaffungErlaubt;
}

Rasse* Rasse::clone() const {
    return new Rasse(this);
}

void Rasse::read(const QDomElement& de) {
    Lernstoff::read(de);
    QDomElement ae = de.firstChildElement("Angeboren");
    if (!ae.isNull()) {
        angeboren = ae.attribute("Name").split(',');
        erschaffungErlaubt = ae.attribute("ErschaffungErlaubt").split(',');
    } else {
        erschaffungErlaubt.clear();
        angeboren.clear();
    }
}

QDomElement Rasse::write(EvaXmlDoc& doc, QDomElement& parent) const {
    QDomElement ret = Lernstoff::write(doc, parent);
    if (angeboren.length() > 0) {
        QDomElement sl = doc.createElement("Angeboren");
        sl.setAttribute("Name", angeboren.join(","));
        sl.setAttribute("ErschaffungErlaubt", erschaffungErlaubt.join(","));
        ret.appendChild(sl);
    }
    return ret;
}

QString Rasse::name_mw(bool frau) const {

    if (frau)
        if (dynamicPropertyNames().contains("Name-w"))  return property("Name-w").toString();
        else                                            return name();
    else
        if (dynamicPropertyNames().contains("Name-m"))  return property("Name-m").toString();
        else                                            return name();
}

int Rasse::statLimit(QString statName) const {
    Q_ASSERT(statName.length() > 0);
    return property(statName.toLatin1()).toInt();
}

//***************** Klasse

Klasse::Klasse(const QDomElement node) : Rasse(node) {
    QStringList gruppen;
    for(QDomElement n = node.firstChildElement(); !n.isNull(); n = n.nextSiblingElement()) {
        QString tag = n.tagName();
        if (tag == "Gruppe")
            gruppen.append(n.attribute("Name"));
    }
    if (oldatt("kannZaubern") != "")
        setProperty("ZAU", true);
    if (gruppen.contains("KAE"))
        setProperty("KAE", true);
    QString apTyp = oldatt("Ausdauer");
    if (apTyp == "ak") setProperty("apbonus", 2);
    else if (apTyp == "k")  setProperty("apbonus", 3);
    else                    setProperty("apbonus", 1);
    if (oldatt("Stand", "Modifikation").length() > 0)
        setProperty("StandBonus", oldatt("Stand", "Modifikation"));
    if (oldatt("Waffen", "Modifikation").length() > 0)
        setProperty("Startwaffen", oldatt("Waffen", "Modifikation"));
    if (oldatt(fl("Rstung"), "Modifikation") == "3")
        setProperty("Startruestung", "KR");
}

Klasse::Klasse(const Klasse* k) : Rasse(k) {
    Q_ASSERT(k->typ() == KLASSE);
    gradEPList = k->gradEPList;
    EPkosten = k->EPkosten;
}

Klasse* Klasse::clone() const {
    return new Klasse(this);
}

int Klasse::gradEP(int grad) const {
    int ret = 0;
    int diff = 0;
    for(int i = 1; i <= grad; i++) {
        if (gradEPList.keys().contains(i))
            diff = gradEPList[i];
        ret += diff;
    }
    return ret;
}

void Klasse::read(const QDomElement& de) {
    Rasse::read(de);
    QDomElement dek = de.firstChildElement("EPKosten");
    for (int i = 0; i < dek.attributes().count(); i++) {
        QDomNode an = dek.attributes().item(i);
        EPkosten[an.nodeName()] = an.nodeValue().toInt();
    }
    dek = de.firstChildElement("Gradanstieg");
    for (int i = 0; i < dek.attributes().count(); i++) {
        QString nn = dek.attributes().item(i).nodeName();
        nn = nn.mid(1);
        gradEPList[nn.toInt()] = dek.attributes().item(i).nodeValue().toInt();
    }
}

QDomElement Klasse::write(EvaXmlDoc& doc, QDomElement& parent) const {
    QDomElement ret = Lernstoff::write(doc, parent);
    QDomElement dek = doc.createElement("EPKosten");
    foreach(QString k, EPkosten.keys())
        dek.setAttribute(k, EPkosten[k]);
    ret.appendChild(dek);
    dek = doc.createElement("Gradanstieg");
    foreach(int g, gradEPList.keys())
        dek.setAttribute("G" + i2s(g), gradEPList[g]);
    ret.appendChild(dek);
    return ret;
}

int Klasse::kostenMult(const Lernstoff* l) const {
    int ret = UNLERNBAR;
    foreach(Assoc a, l->assocs)
        if (a.typ == LE_KOSTEN)                     // Gruppen des Lernstoffs suchen
            ret = qMin(ret, EPkosten[a.sWert]);     // Minimum der Kosten finden
    return ret;
}

QString Klasse::kostenName(const Lernstoff* l, QFont* &font, bool decorate) const {
    QString ret = "";
    font = new QFont("Helvetica");
    font->setItalic(l->gelernt() || kostenMult(l) > 20);
    ret = l->name();
    if (!decorate)
        return ret;
    if (l->typ() != ZAUBER)
        ret += "+" + i2s(l->EW()) ;
    return ret + " (" + i2s(l->kosten()) + ")";
}

//***************** LernList
LernList::~LernList() {
    clear();
}

Lernstoff* LernList::append(Lernstoff* l) {
    if (m_copy)
        l = l->clone();
    QList<Lernstoff*>::append(l);
    return l;
}

bool compLand(const Lernstoff* l1, const Lernstoff* l2) {
    if (*l1 == "Alba") return true;
    if (*l2 == "Alba") return false;
    return l1->name() < l2->name();
}

bool compKlasse(const Lernstoff* l1, const Lernstoff* l2) {
    Q_ASSERT(l1->typ() == KLASSE && l2->typ() == KLASSE);
    Klasse* k1 = (Klasse*)l1;
    Klasse* k2 = (Klasse*)l2;

    if (  k1->isKAE() &&   k2->isZAU())  return true;
    if (  k1->isKAE() &&  !k2->isKAE())  return true;
    if ( !k1->isKAE() &&   k2->isKAE())  return false;
    if ( !k1->isKAE() &&  !k2->isZAU())  return false;
    return k1->name() < k2->name();
}

bool compGelernt(const Lernstoff* l, const Lernstoff* r) {
    if ( r->typ() == ZAUBERGRUPPE) return true;    // Zaubergruppen ans Ende
    if ( l->typ() == ZAUBERGRUPPE) return false;
    if (l->typ() < r->typ()) return true;          // Lerntyp
    if (l->typ() > r->typ()) return false;
    if (l->typ() == WAFFE || l->typ() == STAND) {
        if (l->EW() > r->EW()) return true;        // Fuer Waffen und Staende: EW
        if (l->EW() < r->EW()) return false; 
    }
    return l->name() < r->name();                  // Name
}

bool compStoff(const Lernstoff* l, const Lernstoff* r) {
    if ( r->typ() == ZAUBERGRUPPE) return true;             // Zaubergruppen ganz ans Ende
    if ( l->typ() == ZAUBERGRUPPE) return false;
    if ( l->typ() < r->typ()) return true;                  // Nach Lerntyp sortieren
    if ( l->typ() > r->typ()) return false;
    if ( l->gelernt() && !r->gelernt()) return true;        // bekannt < unbekannt
    if (!l->gelernt() &&  r->gelernt()) return false;
    if (l->typ() == WAFFE) {                                                // bei Waffen
        QString kl = l->property("Kategorie").toString();
        QString kr = r->property("Kategorie").toString();
        if (kl == "Angriff" && kr != "Angriff") return true;                // Angriffswaffen
        if (kl != "Angriff" && kr == "Angriff") return false;
        if (kl == fl("Schuwaffe") && kr != fl("Schuwaffe")) return true;  // Fernwaffen
        if (kl != fl("Schuwaffe") && kr == fl("Schuwaffe")) return false;
        if (kl == "Verteidigung" && kr != "Verteidigung") return true;      // Verteidigunswaffen
        if (kl != "Verteidigung" && kr == "Verteidigung") return false;
    }
    int kl = l->kosten();
    int kr = r->kosten();
    if ( l->gelernt() && (kl < kr)) return true;            // Kosten: bei bekannten teuer zuerst
    if ( l->gelernt() && (kl > kr)) return false;
    if (!l->gelernt() && (kl > kr)) return true;            //         bei unbekannten billig zuerst
    if (!l->gelernt() && (kl < kr)) return false;
    return l->name() < r->name();                           // Name
}

void LernList::sort(LernSort methode) {
    switch (methode) {
    case LES_LAENDER:   qSort(begin(), end(), compLand); break;
    case LES_KLASSEN:   qSort(begin(), end(), compKlasse); break;
    case LES_GELERNT:   qSort(begin(), end(), compGelernt); break;
    case LES_STOFFE:    qSort(begin(), end(), compStoff); break;
    default: Q_ASSERT(false);
    }
}

QString gSortFachgebiet;

bool compFachgebiet(const Lernstoff* l, const Lernstoff* r) {
    if ( l->isGruppe(gSortFachgebiet) && (!r->isGruppe(gSortFachgebiet))) return true;
    if (!l->isGruppe(gSortFachgebiet) && ( r->isGruppe(gSortFachgebiet))) return false;
    return compStoff(l, r);
}

void LernList::sort(QString fachgebiet) {
    if (fachgebiet.length() > 1) {
        gSortFachgebiet = fachgebiet;
        qSort(begin(), end(), compFachgebiet);
    } else
        sort(LES_STOFFE);
};

//***************** Stoffe (Singleton)
// die globalen Variablen
LernHash*    Stoffe::lh  = NULL;

void parseMagusDom(const QDomElement& el, LernHash* lh) {
    Lernstoff* l;
    for(QDomElement n = el.firstChildElement(); !n.isNull(); n = n.nextSiblingElement()) {
        l         = NULL;
        QString s = n.tagName();
        if (s == "Spezies") {
            Rasse* r = new Rasse(n);
            l = r;
            if (r->name_mw(true) != r->name())
                lh->insert(lowerUmlaut(r->name_mw(true)), r);
            if (r->name_mw(false) != r->name())
                lh->insert(lowerUmlaut(r->name_mw(false)), r);
        } else if (s == "Typ") {
            Klasse* k = new Klasse(n);
            l = k;
            if (k->name_mw(true) != k->name())
                lh->insert(lowerUmlaut(k->name_mw(true)), k);
            if (k->name_mw(false) != k->name())
                lh->insert(lowerUmlaut(k->name_mw(false)), k);
        } else
            l = new Lernstoff(n);
        Q_ASSERT(l != NULL);
        lh->insert(l->key(), l);
    }
}

QString getName(const QString& s) {
    if (s.contains("+"))
        return s.left(s.indexOf("+"));
    return s;
}

int getEW(const QString& s) {
    if (s.contains("+"))
        return s.right(s.length() - s.indexOf("+")).toInt();
    return -1;
}

class TEKosten {
public:
    int k[20];
};

void Stoffe::read() {
    if (lh != NULL)
        return;
    gStatNames["St"] = "staerke";
    gStatNames["Gs"] = "geschicklichkeit";
    gStatNames["Gw"] = "gewandheit";
    gStatNames["Ko"] = "konstitution";
    gStatNames["In"] = "intelligenz";
    gStatNames["Zt"] = "zaubertalent";
    gStatNames["Au"] = "aussehen";
    gStatNames["pA"] = "ausstrahlung";
    gStatNames["Wk"] = "willenskraft";

    lh  = new LernHash();
    QString estr;
    int eline, ecol;

#ifdef OLDSTYLE
    // da das Symbol nie definiert wird ist die Semantik von ifndef umgekehrt
    QStringList dontSkip = QStringList() 
        << "Fertigkeiten" 
        << "angeboreneFertigkeiten" 
        << "Typen"
        << "SpeziesListe"
        << "Laender"
        << "Staende"
        << "Sprachen"
        << "Goetter"
        << "Totems"
        << "Waffen-Grundkenntnisse"
        << "Waffen"
        << "Zauber";

    QDirIterator di(".", QStringList() << "*.xml");
    // alle XML-Dateien im Verzeichnis einlesen
    Q_ASSERT(di.hasNext());
    QDomDocument* doc = NULL;
    while (di.hasNext()) {
        di.next();
        doc = new QDomDocument();
        QString fname = di.filePath();
        if (!doc->setContent(new QFile(fname), &estr, &eline, &ecol)) {
            QMessageBox msg;
            msg.setText(estr + "\nDatei: " + fname + "\nZeile: " + i2s(eline) + " Spalte: " + i2s(ecol));
            msg.exec();
            exit(0);
        }
        QDomElement docElem = doc->documentElement(); 
        if (docElem.tagName() == "M4AV-data")
            for(QDomElement n = docElem.firstChildElement(); !n.isNull(); n = n.nextSiblingElement())
                if (dontSkip.contains(n.tagName()))
                    parseMagusDom(n, lh);
    }
    // Jetzt m5tab.txt einlesen
    QFile inf("m5tab\\m5tab.txt");
    if (!inf.open(QIODevice::ReadOnly))
        return;
    QTextStream in(&inf);
    QStringList zauberGruppen = QStringList()
        << "Beherrschen" << "Bewegen" << "Erkennen" << "Erschaffen" << "Formen" << fl("Verndern") << fl("Zerstren") 
        << "Wundertat" << "Dweomer" << "Zauberlied";

    int         state = 0;  // 1 - TE (Fertigkeiten), 2 - LE (Zauber), 3 - Fertigkeiten, 4 - in Fertigkeiten, 5 - Zauber(LE), 6 - Sprueche, 7 - Gradanstieg
    Lernstoff*  l = NULL;
    QString     gruppe;
    int         actTE;
    QHash<int,int>           gradAnstieg;
    QHash<QString,TEKosten>  teKosten;
    while (!in.atEnd()) {
        QStringList sl = QString(in.readLine()).split(":");
        for(int i = 0; i < sl.size(); i++)
            sl[i] = sl[i].simplified();
        if (state < 3 && sl[0] == "TE")
            state = 1;
        if (state < 3 && sl[0] == "LE")
            state = 2;
        if (sl.length() < 3 && fertigkeitsGruppen.contains(sl[0])) {
            gruppe = sl[0];
            state = 3;
        }
        if (sl[0] == "Stufe")
            state = 5;
        if (sl.length() < 3 && zauberGruppen.contains(sl[0])) {
            gruppe = sl[0];
            state = 6;
        }
        if (sl.length() < 3 && sl[0] == "Gradanstieg")
            state = 7;
        switch(state) {
        case 1: { 
            Klasse* k = Stoffe::k(sl[0]);
            if (k == NULL)
                continue;
            for(int i = 1; i < sl.length(); i++)
                k->EPkosten[fertigkeitsGruppen[i - 1]] = sl[i].toInt();
            break;
                }
        case 2: {   
            Klasse* k = Stoffe::k(sl[0]);
            if (k == NULL)
                continue;
            for(int i = 1; i < sl.length(); i++)
                k->EPkosten[zauberGruppen[i - 1]] = sl[i].toInt();
            sl.clear();
            break;
                }
        case 3: {
            if (sl.length() < 4) {
                if (sl.length() == 0 || sl[0].length() < 2)
                    state = 4;
                continue;
            }
            if (sl[0] == "TE") {
                actTE = sl[1].toInt();
                teKosten.clear();
                continue;
            }
            TEKosten tek;
            memset(tek.k, 0, sizeof(int) * 20);
            for(int i = 1; i < sl.count(); i++)
                tek.k[actTE + i - 1] = sl[i].toInt();
            teKosten[sl[0]] = tek;
            break;
                }
        case 4: {
            if (sl.length() < 4) {
                if (sl.length() == 0 || sl[0].length() < 2)
                    state = 3;
                continue;
            }
            for(int i = 0; i < (sl.length() - 2) / 2; i++) {
                QString f = getName(sl[3 + i * 2]);
                l = Stoffe::l(f);
                l->setEW(getEW(sl[3 + i * 2]));
                Assoc a;
                a.typ = LE_KOSTEN;
                a.sWert = gruppe;
                a.iWert = sl[1].toInt();
                l->assocs.append(a);
                memcpy(l->TEkosten, teKosten[sl[0]].k, sizeof(int) * 20);
            }
            break;
                }
        case 5: {
            if (sl[0] != "LE")
                continue;
            teKosten.clear();
            for(int i = 0; i < 13; i++)
                teKosten["Zauber"].k[i] = sl[i].toInt();
            break;
                }
        case 6: {
            if (sl.length() < 2)
                continue;
            QString zn = sl[0];
            Lernstoff* l = Stoffe::l(sl[0]);
            if (l == NULL)
                continue;
            int stufe = sl[1].toInt();
            l->setProperty("Stufe", stufe);
            Assoc a;
            a.typ = LERNPREREQ;
            a.iWert = teKosten["Zauber"].k[stufe];
            a.sWert = gruppe;
            l->assocs.append(a);
            break;
                }
        case 7: {
            if (sl.length() == 2)
                gradAnstieg[sl[0].toInt()] = sl[1].toInt();
            break;
                }
        default: Q_ASSERT(false); break;
        }   // switch(state)
    }   // while (!in.atEnd())
    foreach(l, Stoffe::l(KLASSE)) {
        Klasse* k = qobject_cast<Klasse*>(l);
        k->gradEPList = gradAnstieg;
    }
    write();
    writeZauberDesc();
#else   // newstyle
    QDirIterator    di(".", QStringList() << "*.xml");
    EvaXmlDoc       doc("");
    while (di.hasNext()) {
        di.next();
        if (!doc.read(di.filePath()))
            continue;
        if (doc.root.tagName() == "Lernstoffe")
            for(QDomElement n = doc.documentElement().firstChildElement(); !n.isNull(); n = n.nextSiblingElement()) {
                Lernstoff *l = NULL;
                QString typ = n.attribute("Typ");
                if      (typ == "Rasse")
                    l = new Rasse();
                else if (typ == "Klasse")
                    l = new Klasse();
                else
                    l = new Lernstoff();
                l->read(n);
                lh->insert(l->key(), l);
                if (l->typ() == RASSE || l->typ() == KLASSE)
                    for(int i = 0; i < 2; i++) {
                        QString nmw = lernKey(((Rasse*)l)->name_mw(i == 0));
                        if (!lh->keys().contains(nmw))
                            lh->insert(nmw, l);
                    }
            }
        if (doc.root.tagName() == "Zauberdesc")
            for(QDomElement n = doc.documentElement().firstChildElement(); !n.isNull(); n = n.nextSiblingElement()) {
                QString s = n.attribute("Name");
                Lernstoff* l = Stoffe::l(s);
                if (l == NULL)
                    continue;
                s = n.attribute("Beschreibung");
                l->setProperty("Beschreibung", s);
                QString ss = l->name();
                if (l->property("Stufe").toInt() < 3)
                    int i = 0;
            }
    }
#endif  // newstyle
    if (lh->size() < 10) {
        QMessageBox msg;
        msg.setText("Keine Datendateien (*.m5) im richtigen XML-Format gefunden");
        msg.exec();
    }
    Lernstoff *ls = Stoffe::l("Schwingenkeule");
    QString as = ls->property("Beschreibung").toString();
    if (as.length() < 2)
        int i = 0;
}

void Stoffe::write() {
    EvaXmlDoc doc("Lernstoffe");
    for(int i = 0; i < ZAUBER; i++) {
        LernList    ll(false);
        foreach (Lernstoff* l, lh->values()) {
            if (l->typ() != i)
                continue;
            bool dup = false;
            foreach(const Lernstoff* gl, ll)
                dup = dup ||*gl == *l;
            if (!dup)
                ll.append(l);
        }
        ll.sort(LES_STOFFE);
        foreach(const Lernstoff* l, ll)
                l->write(doc, doc.root);
    }
    // write the domDocument to the file
    doc.write("m5kodex.xml");

    EvaXmlDoc doc2("Lernstoffe");
    for(int i = ZAUBER; i < LAST_LERNSTOFF; i++) {
        LernList    ll(false);
        foreach (Lernstoff* l, lh->values()) {
            if (l->typ() != i)
                continue;
            bool dup = false;
            foreach(const Lernstoff* gl, ll)
                dup = dup ||*gl == *l;
            if (!dup)
                ll.append(l);
        }
        ll.sort(LES_STOFFE);
        foreach(const Lernstoff* l, ll)
            if (l->assocs.length() > 0)
                l->write(doc2, doc2.root);
    }
    // write the domDocument to the file
    doc2.write("m5arkanum.xml");

}

void Stoffe::writeZauberDesc() {
    EvaXmlDoc doc("Zauberdesc");
    QDomElement notiz = doc.createElement("Weitergabe");
    notiz.setAttribute("Text", fl("Diese Datei und ihr Inhalt drfen nicht weitergegeben oder ffentlich zugnglich gemacht werden"));
    doc.root.appendChild(notiz);
    LernList    ll(false);
    foreach (Lernstoff* l, lh->values()) {
        if (l->typ() != ZAUBER)
            continue;
        bool dup = false;
        foreach(const Lernstoff* gl, ll) {
            dup = dup ||*gl == *l;
            if (dup)
                break;
        }
        if (!dup)
            ll.append(l);
    }
    foreach(const Lernstoff* l, ll) {
        if (l->property("Beschreibung").toString().length() < 2)
            continue;
        QDomElement le = doc.createElement("Desc");
        le.setAttribute("Name", l->name());
        le.setAttribute("Beschreibung", l->property("Beschreibung").toString());
        doc.root.appendChild(le);
    }
    doc.write("m5zauberdesc.xml");
}

Lernstoff* Stoffe::l(const QString& name) {
    if (name == "")
        return NULL;
    return lh->value(lowerUmlaut(name));
}

Rasse* Stoffe::r(const QString& name) {
    Rasse* ret = (Rasse*)l(name);
    if (ret->typ() != RASSE)
        return NULL;
    return ret;
}

Klasse* Stoffe::k(const QString& name) {
    Klasse* ret = (Klasse*)l(name);
    if (ret == NULL || ret->typ() != KLASSE)
        return NULL;
    return ret;
}


const QStringList Stoffe::keys() {
    return lh->keys();
}

LernList Stoffe::l(LernstoffTypen eTyp) {
    LernList ret(false);
    foreach(Lernstoff* l, lh->values()) {
        if (l->typ() == eTyp && !ret.contains(l))
            ret.append(l);
    }
    return ret;
}

bool s2bool(const QString s) {
    if (s.toUpper() == "TRUE") return true;
    if (s == "1") return true;
    if (s.toUpper() == "FALSE") return false;
    if (s == "0") return false;
    Q_ASSERT(false);
    return false;
}

QString lowerUmlaut(const QString& q) {
    QByteArray  ba = q.toLatin1();
    char        ret[1024];
    char*       s = ba.data();
    char*       d = ret;
    while (*s) {
        char c = *s;
        if ('A' <= c && c <= 'Z')
            c = c + 'a' - 'A';
        switch(c) {
        case 'A': case '': *d++ = 'a'; *d++ = 'e'; break;
        case '': case '': *d++ = 'o'; *d++ = 'e'; break;
        case '': case '': *d++ = 'u'; *d++ = 'e'; break;
        case '': *d++ = 's'; *d++ = 's'; break;
        case ' ': break;
        default: *d++ = c;
        }
        s++;
    }
    *d = 0;
    return ret;
}

QString lernKey(const QString& s) {
    QString ret = lowerUmlaut(s);
    int i = ret.indexOf(':');
    if (i > -1)
        ret.truncate(i);
    i = ret.indexOf('+');
    if (i > -1)
        ret.truncate(i);
    return ret;
}

int roll (int n, int w) {
    int res = 0;
    for (int i = 0; i < n; i++)
        res += (qrand() % w) + 1;
    return res;
}

QString statName(const QString& shortStatName) {
    Q_ASSERT(gStatNames.keys().contains(shortStatName));
    return gStatNames.value(shortStatName);
}

QString shortStatName(const QString& statName) {
    foreach(QString k, gStatNames.keys())
        if (gStatNames[k] == statName)
            return k;
    return "";
}

