#ifndef LERNSTOFF_H
#define LERNSTOFF_H

#include "assoc.h"

enum LernstoffTypen {
    GOTT, TOTEM, LAND, STAND, RASSE, KLASSE,                        // die Typen < ANGEBOREN sind nur 1x lernbar
    ANGEBOREN, ZAUBERGRUPPE, WAFFENGRUPPE, FERTIGKEIT, WAFFE, ZAUBER,
    LAST_LERNSTOFF
};

typedef enum {LK_NONE, LK_ROLLE, LK_VOLL, LK_LAST} eLernkostentyp;

class Lernstoff : public QObject {
    Q_OBJECT

public:
    Lernstoff();
    Lernstoff(const Lernstoff* l);
    Lernstoff(const QDomElement node);
    ~Lernstoff();

    virtual Lernstoff* clone() const;                               // Kopie von this (von der richtigen Klasse)
    
    QString         key() const {return m_key;};                    // key in der Lernstoff-Hashtabelle (wird von setName gesetzte)
    QStringList     zusaetze() const;
    void            setKosten(eLernkostentyp kosten);               // setzt kosten. LE wenn nicht gelernt, TE und neuen EW setzen wenn steigerbar und gelernt, UNLERNBAR sonst
    bool            isGruppe(const QString& gruppe) const;          // gruppe kann ,-getrennte Liste sein, true wenn Lernstoff in einer der Fertigkeits- oder Zaubergruppen ist

    int             rollProp(const QString& prop) const;            // Eine property mit Wuerfel-Information auswuerfeln
    QStringList     landesSprachen(bool nachbarn = false) const;    // Hier lernbare (Nachbar-)Sprachen (leer wenn typ() != LAND)

    virtual QDomElement write(EvaXmlDoc& doc, QDomElement& parent) const;
    virtual void    read(const QDomElement& de);

    bool operator==(Lernstoff const& rhs) const;
    bool operator==(QString   const& rhs) const;
    bool operator!=(Lernstoff const& rhs) const;
    bool operator!=(QString   const& rhs) const;

    AssocList       assocs;
    int             TEkosten[20];

    eLernkostentyp  m_lernkostentyp;

protected:
    void            setPropAttr(const QString& propName, const QString& attrName = "", const QString& nodeName = "");
    QString         oldatt(const QString attrName, const QString node1 = "", const QString node2 = "") const; // Attribute attName lesen, davor bis zu zwei Nodes tiefer gehen

    QDomElement     m_node;
    QStringList     m_zusaetze;

// Ab hier: Properties und Verwaltung dazu
public:
    Q_PROPERTY(QString          name            READ name               WRITE setName)
    Q_PROPERTY(int              typ             READ typ                WRITE setTyp)
    Q_PROPERTY(int              PP              READ PP                 WRITE setPP)
    Q_PROPERTY(int              EW              READ EW                 WRITE setEW)
    Q_PROPERTY(int              zusatz          READ zusatz             WRITE setZusatz)
    Q_PROPERTY(bool             gelernt         READ gelernt            WRITE setGelernt)
    Q_PROPERTY(int              kosten          READ kosten             WRITE setKosten)

    QString         name() const;
    void            setName(const QString& name);               // setzt auch m_key
    int             typ() const {return m_typ;};
    void            setTyp(int typ) {Q_ASSERT(typ >= 0 && typ <= LAST_LERNSTOFF); m_typ = typ;};
    int             PP() const {return m_pp;};
    void            setPP(int pp) {m_pp = pp;};
    int             EW() const {return m_EW;};
    void            setEW(int EW) {m_EW = EW;};
    int             zusatz() const {return m_zusatz;};
    void            setZusatz(int zusatz) {m_zusatz = zusatz;};
    bool            gelernt() const {return m_gelernt;};
    void            setGelernt(bool gelernt) {m_gelernt = gelernt;};
    int             kosten() const {return m_kosten;};
    void            setKosten(int kosten) {m_kosten = kosten;};

protected:
    void setDynProp(const QString& name, const QVariant& value);    // dynamic properties werden fuer die meisten statischen Attribute verwendet

private:
    QString         m_name;
    QString         m_key;
    int             m_typ;
    bool            m_gelernt;
    int             m_EW;
    int             m_pp;
    int             m_kosten;
    int             m_zusatz;
};

//************************** Rasse 

class Rasse : public Lernstoff {
    Q_OBJECT

public:
    Rasse() : Lernstoff() {angeboren.clear();};
    Rasse(const QDomElement node);
    Rasse(const Rasse* r);

    virtual Rasse* clone() const;           // Kopie von this (von der richtigen Klasse)
    
    QString name_mw(bool frau) const;
    int     statLimit(QString statName) const;

    virtual QDomElement write(EvaXmlDoc& doc, QDomElement& parent) const;
    virtual void    read(const QDomElement& de);

    QStringList     angeboren;              // automatisch angeborene Fertigkeiten dieser Klasse
    QStringList     erschaffungErlaubt;     // zusaetzliche Einschraenkungen bei Erschaffung (z.B. Ueberleben:Wald)
};

//************************** Klasse 

class Klasse : public Rasse {
    Q_OBJECT

public:
    Klasse() : Rasse() {};
    Klasse(const QDomElement node);
    Klasse(const Klasse* k);

    virtual Klasse* clone() const;                          // Kopie von this (von der richtigen Klasse)

    virtual QDomElement write(EvaXmlDoc& doc, QDomElement& parent) const;
    virtual void    read(const QDomElement& de);
    int     kostenMult(const Lernstoff* l) const;           // Lernkosten-Multiplikator (10 - 50)
    QString kostenName(const Lernstoff* l, QFont* &font, bool decorate = true) const; // name von l mit EW und Kosten (falls decorate), setzt Font abhaengig von Affinitaet
    int     gradEP(int grad) const;                         // return max. Zahl von GFP in diesem Grad

    bool isKAE() {return property("KAE").toBool();};        // Kaempfer incl. Ba und Ok
    bool isZAU() {return property("ZAU").toBool();};        // Zauberer incl. Ba und Ok

    QHash<QString,int>      EPkosten;                       // key: Gruppe, value: EP
    QHash<int, int>         gradEPList;                     // key: Grad bei dem sich was aendert, value: neues ES-Inkrement
};

//************************** LernHash und LernList

typedef QHash<QString, Lernstoff*>  LernHash;

typedef enum LernSort {     // Sortiermethoden fuer LernListen
    LES_LAENDER,            // alphabetisch, Alba garantiert an erster Stelle
    LES_KLASSEN,            // Erst KAE, dann ZKAE, dann ZAU, ansonsten alphabetisch
    LES_GELERNT,            // 1. Typ, 2. bei Waffen: EW, 3. alphabetisch
    LES_STOFFE,             // 1. Typ, 2. bekannt, 3. Waffen-Typ, 4. Kosten, 5. alphabetisch
    LES_LAST
};

class LernList : public QList<Lernstoff*> {

public:
    LernList(bool copy) {m_copy = copy;};       // copy gibt an ob die Liste aus Kopien besteht oder nur aus Zeigern
    ~LernList();

    Lernstoff* append(Lernstoff* l);            // fuegt neues Element ein, gibt Zeiger darauf zurueck (ggf. ist das eine Kopie)

    void sort(LernSort methode);
    void sort(QString fachgebiet);              // wie LES_STOFFE, aber diejenigen mit gruppe == fachgebiet zuerst

private:
    bool    m_copy;
};

extern QString gLernstoffTypNamen[LAST_LERNSTOFF];
//extern QHash<QString, QString> gStatNames;  // gStatNames[Abk] = langer Name

// Singleton fuer die globale LernList
class Stoffe {
public:
    static Lernstoff* l(const QString& name);
    static Rasse*     r(const QString& name);
    static Klasse*    k(const QString& name);
    static LernList l(LernstoffTypen eTyp);  // Liste von Stoffen des passenden Typs
    static const QStringList keys();
    static void read();                         // XML einlesen
    static void write();                        // XML schreiben (fliegt noch raus)
    static void writeZauberDesc();              // Datei mit Zauber-Beschreibungstexten schreiben

private:
    Stoffe() {};
    static LernHash*        lh;
};

QString lowerUmlaut(const QString& s);
// lower case und Umlaute ersetzen, Blanks strippen

QString lernKey(const QString& s);
// lowerUmlaut und Zusaetze und EW entfernt

extern QHash<QString, QString> gStatNames;      // gStatNames[Abk] = langer Name

QString shortStatName(const QString& statName); // Geschicklichkeit -> Ge

int roll (int n, int w);

extern QStringList fertigkeitsGruppen;          // Alltag, Freiland, Halbwelt etc.

#endif // LERNSTOFF_H