#include "neu3ap.h"
#include "neuwizard.h"

Neu3Ap::Neu3Ap(QWidget *parent)
    : QWizardPage(parent)
{
    setupUi(this);
    LernList staende = Stoffe::l(STAND);
    staende.sort(LES_GELERNT);          // Nach EW sortieren (Staende missbrauchen den EW fuer die Reihenfolge)
    foreach(Lernstoff* l, staende)
        stand->addItem(l->name());
    fw = new Bogen(this);
}

bool Neu3Ap::validatePage() {
    FIGUR->setWidgetValue("ausdauer", ausdauer->text());
    FIGUR->setWidgetValue("leben", leben->text());
    FIGUR->setWidgetValue("glaube", glaube->currentText());
    FIGUR->setWidgetValue("gott", gott->currentText());
    FIGUR->setWidgetValue("stand", stand->currentText());
    FIGUR->lerne("Sprache:" + muttersprache->currentText());
    FIGUR->lerne("Sprache:" + zweitsprache->currentText());
    FIGUR->lerneTypisch();
    return QWizardPage::validatePage();
}

void Neu3Ap::initializePage() {
    QSettings settings;

    glaube->clear();
    Klasse* k = FIGUR->klasse();
    if (*k == "PB" || *k == "PS" || *k == "Or")
        glaube->addItem(fl("Gtter"));
    else if (*k == "Sc" || *k == "Tm")
        glaube->addItem("schamanistisch");
    else if (*k == "Dr" || *(FIGUR->rasse()) == "Elf" || FIGUR->rasse()->name().contains("gnom"))
        glaube->addItem("druidisch");
    else if (*k == "Be")
        glaube->addItem(fl("gleichgltig"));
    else {
        glaube->addItem(fl("gleichgltig"));
        glaube->addItem(fl("Gtter"));
        glaube->addItem("druidisch");
        glaube->addItem("schamanistisch");
    }
    on_glaube_currentIndexChanged(0);

    QStringList fsprachen = FIGUR->muttersprachen();
    QStringList sprachen = Stoffe::l("Sprache")->zusaetze();
    if (settings.value("Anfangssprachen", true).toBool())
        sprachen = fsprachen;
    muttersprache->clear();
    foreach(QString s, sprachen)
        muttersprache->addItem(s + "+12");
    muttersprache->setCurrentIndex(muttersprache->findText(fsprachen[0] + "+12"));

    fsprachen = FIGUR->zweitsprachen();
    if (settings.value("Anfangssprachen", true).toBool())
        sprachen = fsprachen;
    zweitsprache->clear();
    foreach(QString s, sprachen)
        zweitsprache->addItem(s + "+12");
    zweitsprache->setCurrentIndex(zweitsprache->findText(fsprachen[0] + "+12"));
       on_reroll_clicked();
}

void Neu3Ap::on_glaube_currentIndexChanged(const QString& text) {
    if (text == "")
        return;
    gott->clear();
    gott->setEnabled(false);
    label_gott->setText("");
    if (text == fl("gleichgltig") || text == "druidisch")
        return;
    if (text == "schamanistisch") {
        label_gott->setText("Totem");
        foreach(const Lernstoff* l, Stoffe::l(TOTEM))
                gott->addItem(l->name());
        if (*FIGUR->klasse() != "Sc")
            gott->insertItem(0, "");    // kein Totem geht nur bei nicht-Schamanen
        gott->setEnabled(true);
    }
    if (text == fl("Gtter")) {
        label_gott->setText("Gott");
        foreach(const Lernstoff* l, FIGUR->lernbar(GOTT, LK_NONE))
            gott->addItem(l->name());
        gott->setEnabled(true);
    }
    gott->model()->sort(0);
    gott->setCurrentIndex(0);
}

void Neu3Ap::on_reroll_clicked() {
    fw->update(FIGUR);
    FIGUR->setAusdauer(0);
    ausdauer->setText(i2s(FIGUR->rollAP()));
    leben->setText(i2s(roll(1,3) + FIGUR->konstitution() / 10 + 7 + FIGUR->rasse()->property("LPbonus").toInt()));
    int m = 0;
    foreach(Lernstoff* l, FIGUR->gelerntList)
        m += l->property("StandBonus").toInt();
    int a = roll(1, 100) + m;
    m = FIGUR->rasse()->property("StandMin").toInt();
    if (m > 0)
        a = qMax(a, m);
    m = FIGUR->rasse()->property("StandMax").toInt();
    if (m > 0)
        a = qMin(a, m);
         if (a < 11) stand->setCurrentIndex(0);
    else if (a < 51) stand->setCurrentIndex(1);
    else if (a < 91) stand->setCurrentIndex(2);
    else             stand->setCurrentIndex(3);
}