#include "neu4beruf.h"
#include "neuwizard.h"

Neu4Beruf::Neu4Beruf(const LernList& anList, QWidget *parent) : angeborenList(false), QWizardPage(parent)
{
    setupUi(this);
    fw = new Bogen(this);
    angeborenList = anList;
}

Neu4Beruf::~Neu4Beruf() {
}

void Neu4Beruf::initializePage() {
    fw->update(FIGUR);
    merkmale->setText(FIGUR->rasse()->name());

    spezialisierung->clear();
    spezialisierung->setEnabled(FIGUR->klasse()->property("Spezial").isValid());
    if (spezialisierung->isEnabled()) {
        QStringList spez = FIGUR->klasse()->property("Spezial").toString().split(',');
        spezialisierung->setEnabled(spez.count() < 2 || spez[0].toInt() == 0 || spez[1] != "Waffe");
        if (spezialisierung->isEnabled())
            foreach(QString s, spez)
                spezialisierung->addItem(s);
    }
    angeboren2->clear();
    angeboren2->setEnabled(false);
    on_reroll_clicked();
    AssocList fGruppen = FIGUR->fGruppenList();
        
    foreach(QWidget* w, this->findChildren<QWidget*>(QRegExp("fach_"))) {
        // die Fertigkeitsgruppen initialisieren
        QStringList wnl = w->objectName().split("_");
        int n = wnl[1].toInt();
        if (n > 0) {
            QLineEdit* le = qobject_cast<QLineEdit*>(w);
            if (n > fGruppen.count())
                le->setText("");
            else
                le->setText(i2s(fGruppen[n - 1].iWert));
            continue;
        }
        Q_ASSERT(wnl.size() == 3);
        n = wnl[2].toInt();
        QComboBox* cb = qobject_cast<QComboBox*>(w);
        cb->clear();
        foreach(QString s, fertigkeitsGruppen) 
            cb->addItem(s);
        if (FIGUR->klasse()->isZAU())
            cb->addItem("Zauber");
        if (n <= fGruppen.size())
            if (!fertigkeitsGruppen.contains(fGruppen[n - 1].sWert))    // keine Fertigkeitsgruppe - muss wohl Zaubern sein
                cb->setCurrentIndex(cb->count() - 1);
            else
                cb->setCurrentIndex(fertigkeitsGruppen.indexOf(fGruppen[n - 1].sWert));
        else {
            cb->insertItem(0, "");
            cb->setCurrentIndex(0);
        }
    }
    qobject_cast<NeuWizard*>(wizard())->addSeiten(fGruppen);
    update();
}

void Neu4Beruf::rollAngeboren() {
    int r = roll(1, 100);
    if (r == 100 && angeboren2->isEnabled())            // 2x nacheinander 100 wuerfeln gibt's nicht
        r = roll(1, 80) + 15;           // in dem Fall verbieten wir aber die "komischen" Ergebnisse
    angeboren->setCurrentIndex(0);
    for(int i = 1; i < angeboren->count() - 1; i++) {   // diverse -1 wegen den zusaetzlichen
        const Lernstoff* l = angeborenList[i - 1];      // Elementen ("keine" u. "freie Wahl")
        if (l->property("Min").toInt() > r) {
            angeboren->setCurrentIndex(i - 1);
            break;
        }
    }
    if (angeboren->currentIndex() == 0)     // keine Min-Property war groesser => 96 oder mehr (oder so)
        if (r == 100)
            angeboren->setCurrentIndex(angeboren->count() - 1);
        else
            angeboren->setCurrentIndex(angeboren->count() - 2);
}

void Neu4Beruf::on_angeboren_currentIndexChanged(int index ) {
    if (index < 1 || index < angeboren->count() - 1)
        return;     // index war kleiner als count() - 1, also nix besonderes
    // index ist das letzte Listenelement, also 100, d.h. nachwuerfeln und freie Wahl
    if (!angeboren2->isEnabled()) {
        angeboren2->setEnabled(true);   // freie Wahl einschalten und befuellen
        foreach(Lernstoff* l, angeborenList)
            if (l->EW() > 0 && l->property("Min").toInt() > 0)
                angeboren2->addItem(l->name() + FIGUR->EWStr(l));
        angeboren2->insertItem(0, "keine");
    }
    rollAngeboren();                    // nachwuerfeln
}

void Neu4Beruf::on_reroll_clicked() {
    angeboren->clear();
    foreach(Lernstoff* l, angeborenList)
        if (l->property("Min").toInt() > 0)
            angeboren->addItem(l->name() + FIGUR->EWStr(l));
    angeboren->insertItem(0, "keine");
    angeboren->insertItem(100, "freie Wahl & nochmal");
    angeboren2->clear();
    angeboren2->setEnabled(false);
    angeboren->setCurrentIndex(0);
    rollAngeboren();
}

bool Neu4Beruf::validatePage() {
    FIGUR->setWidgetValue("spezialisierung", spezialisierung->currentText());
    FIGUR->setWidgetValue("merkmale", merkmale->text());
    FIGUR->lerneAngeboren(angeboren->currentText(), angeboren2->currentText());

    return QWizardPage::validatePage();
}