#include "neu5fertigkeiten.h"
#include <QInputDialog>

class KenntnisLwi : public QListWidgetItem {
public:
    KenntnisLwi(const Figur* f, Lernstoff* ls, QListWidget* view, bool decorate = true);
    ~KenntnisLwi();

    int kosten();
    QString name() {return l->name();};

    QFont*          font;
    Lernstoff*      l;
    const Figur*    figur;
};

KenntnisLwi::KenntnisLwi(const Figur* f, Lernstoff* ls, QListWidget* view, bool decorate) : QListWidgetItem(f->klasse()->kostenName(ls,font, decorate), view) {
    // Obacht: font wird von kostenName() neu angelegt (muss so komisch sein wg. constructor chaining)
    l = ls->clone();
    figur = f;
    if (f->isSpezialisierung(l))
        font->setWeight(80);
    setFont(*font);
}

KenntnisLwi::~KenntnisLwi() {
    delete l;
    delete font;
}

int KenntnisLwi::kosten() {
    figur->setKosten(l, LK_VOLL);
    return l->kosten();
}

Neu5Fertigkeiten::Neu5Fertigkeiten(const Assoc& a, QWidget *parent)
    : QWizardPage(parent)
{
    setupUi(this);
    gruppe = a;
    if (a.sWert.count(',') > 2)
        label_gruppe->setText(fl("LE fr Zaubersprche"));
    else
        label_gruppe->setText(fl("LE fr ") + a.sWert);
    fw = new Bogen(this);
}

Neu5Fertigkeiten::~Neu5Fertigkeiten() {
}

void Neu5Fertigkeiten::setLernbar(int lernpunkte) {
    int actLE = label_le->text().toInt() - lernpunkte;
    label_le->setText(i2s(actLE));
    LernstoffTypen lTyp;
    if (gruppe.sWert == "Zauber")
        lTyp = ZAUBER;
    else if (gruppe.sWert == "Waffen")
        lTyp = WAFFENGRUPPE;
    else
        lTyp = FERTIGKEIT;
    lernbar->clear();
    LernList ll = FIGUR->lernbar(lTyp, LK_NONE);
    foreach(Lernstoff* l, ll) {
        FIGUR->setKosten(l, LK_VOLL);
        if ( (lTyp != ZAUBER && !l->isGruppe(gruppe.sWert))
        ||   l->kosten() > actLE
        ||   FIGUR->gelernt(l->name()) != NULL)
            ll.removeAll(l);
    }
    ll.sort(FIGUR->spezialisierung());
    foreach(Lernstoff* l, ll)
        lernbar->addItem(new KenntnisLwi(FIGUR, l, lernbar));
}

void Neu5Fertigkeiten::checkGelernt() {
    int kosten = 0;
    foreach(QListWidgetItem* lw, gelernt->findItems("*", Qt::MatchWildcard)) {
        KenntnisLwi* klwi = (KenntnisLwi*) lw;
        QString name = klwi->l->name();
        if (!FIGUR->gelernt(name)) {
            kosten += klwi->l->kosten();
            delete klwi;
        }
    }
    setLernbar(-kosten);            // lernbar neu aufbauen (gesparte Lernpunkte anpassen)
}

void Neu5Fertigkeiten::initializePage() {
    fw->update(FIGUR);
    label_alltag_le->setText(i2s(SPARLE));
    gelernt->clear();
    known.clear();
    foreach(Lernstoff* l, FIGUR->gelerntList)
        if (l->isGruppe(gruppe.sWert)) {
            known.append(l->name());
            gelernt->addItem(new KenntnisLwi(FIGUR, l, gelernt, false));
        }
    if (gruppe.sWert == "Alltag")
        label_le->setText(label_alltag_le->text());
    else
        label_le->setText(i2s(gruppe.iWert));
    setLernbar(0);
}

bool Neu5Fertigkeiten::validatePage() {
    if (label_le->text().toInt() > 0)
        SPARLE++;
    return QWizardPage::validatePage();
}

void Neu5Fertigkeiten::on_lerne_clicked() {
    KenntnisLwi* lwi = (KenntnisLwi*)lernbar->takeItem(lernbar->currentRow());
    if (lwi == NULL)
        return;
    int kosten = lwi->kosten();
    Lernstoff* l = NULL;
    if (lwi->l->zusaetze().count() > 1 && FIGUR->rasse()->erschaffungErlaubt.join(",").contains(lwi->name())) {
        QStringList erlaubt;
        QString res;
        foreach(QString e, FIGUR->rasse()->erschaffungErlaubt)
            if (e.contains(lwi->name()))
                erlaubt.append(e.mid(e.indexOf(":") + 1));
        bool ok = erlaubt.count() > 0;
        if (erlaubt.count() == 1)
            res = erlaubt[0];
        else if (ok)
            res = QInputDialog::getItem(NULL, "Fertigkeit spezifizieren", lwi->name() + ":", erlaubt, 0, false, &ok);
        res = lwi->name() + ":" + res;
        if (ok && !FIGUR->gelernt(res))
            l = FIGUR->lerne(res);
    } else
        l = FIGUR->lerne(lwi->l);
    delete lwi;
    if (l == NULL)
        setLernbar(0);
    else {
        l->setKosten(kosten);
        gelernt->addItem(new KenntnisLwi(FIGUR, l, gelernt));
        setLernbar(kosten);
    }
}

void Neu5Fertigkeiten::on_vergesse_clicked() {
    KenntnisLwi* lwi = (KenntnisLwi*)gelernt->item(gelernt->currentRow());
    if (lwi == NULL || known.contains(lwi->name()))
        return;
    FIGUR->vergesse(lwi->l->name());
    checkGelernt();
}
