// Code to create a QComboBox where multiple items can be selected and the items
// are shown in a tree.
// Written by Corwin Joy
// This code is hereby placed in the Public Domain.
// This code comes with no warranty of any kind, use at your own risk.

#pragma once
#include <QComboBox.h>

class QStandardItemModel;

class NeuMultiCombo : public QComboBox
{
Q_OBJECT

public:
  NeuMultiCombo(int maxCheck, QModelIndex defaultRootIdx = QModelIndex(), QWidget* parent = 0);
  void setRoot(QModelIndex defaultRootIdx)
  {
    defaultRootIndex = defaultRootIdx;
  }

  bool eventFilter(QObject* object, QEvent* event);
  virtual void showPopup();
  virtual void hidePopup();
  virtual void addItem(const QString &text, const QVariant &userData = QVariant());
  QModelIndexList currentIndexes() const;
  QModelIndexList getChildren(QModelIndex current) const;
  QVariant itemData (QModelIndex index, int role = Qt::UserRole + 1 ) const; 
  QModelIndexList findData (const QVariant & data, int role = Qt::UserRole + 1, Qt::MatchFlags flags = Qt::MatchExactly | Qt::MatchCaseSensitive ) const; 
  QModelIndexList findText (const QString & text, Qt::MatchFlags flags = Qt::MatchExactly | Qt::MatchCaseSensitive ) const; 
  QStandardItemModel *model() const;
  
public Q_SLOTS:
  void setCurrentIndexes(QModelIndexList indexes); 

Q_SIGNALS:
  void currentIndexChanged(QModelIndexList indexes);

protected:
  // private functions
  void UpdateSelectionString();
  void ToggleItem(QModelIndex const &index);
  void setCheckState(QModelIndex current, bool bForceChildrenChecked, 
    bool bNeverForceChildren, bool &bChildrenChecked, bool &bChildrenUnChecked);
  QModelIndexList getCheckedChildren(QModelIndex current) const;

private:
  // functions forbidden from base.  these don't make sense in our derived class
  int currentIndex () const {};
  void currentIndexChanged(int index); 
  void currentIndexChanged ( const QString & text );
  void activated ( int index ) ; 
  void activated ( const QString & text );  
  QVariant itemData ( int index, int role = Qt::UserRole ) const; 
  QIcon itemIcon ( int index ) const ;
  QString itemText ( int index ) const;

  // member variables
  bool skipNextHide;

protected:
  QModelIndex   defaultRootIndex;
  QString       strSelected;
  int           checkMax;       // maximum number of checkable items
  int           checkCount;     // current number of items checked (<= checkMax)
};
