#include "neuwizard.h"
#include "neu1rasse.h"
#include "neu2typ.h"
#include "neu3ap.h"
#include "neu4beruf.h"
#include "neu5fertigkeiten.h"
#include "neu6dinge.h"

NeuWizard::NeuWizard(Figur** f, QWidget *parent) : QWizard(parent) {
    setupUi(this);
    oriFigur = f;
    actFigur = new Figur();
    actFigur->setGrad(1);

    figuren.clear();
    addPage(new Neu1Rasse(this));
    addPage(new Neu2Typ(this));
    addPage(new Neu3Ap(this));
    berufId = addPage(new Neu4Beruf((*f)->getAngeboren(), this));
}

NeuWizard::~NeuWizard() {
    foreach(Figur* f, figuren)
        if (f != NULL)
            delete f;
    delete actFigur;
}

int NeuWizard::nextId() const {
    int n = QWizard::nextId();
    return n;
}

Figur* NeuWizard::figur() const {
    return actFigur;
}

void NeuWizard::initializePage(int id) {
    if (figuren.keys().contains(id))
        delete figuren[id];
    figuren[id] = new Figur(actFigur);
    QWizard::initializePage(id);
}

void NeuWizard::addSeiten(const AssocList& gList) {
    foreach(int p, pageIds())
        if (p > berufId)
            removePage(p);
    for (int i = 1; i < gList.count(); i++)
        addPage(new Neu5Fertigkeiten(gList[i], this));
    addPage(new Neu5Fertigkeiten(gList[0], this));      // allgemeine Fertigkeiten ganz an den Schluss
    sparLE = gList[0].iWert;                            // "gesparte" LE initialisieren
    addPage(new Neu6Dinge(this));
}

void NeuWizard::cleanupPage(int id) {
    delete actFigur;
    actFigur = new Figur(figuren[id]);
    QWizard::cleanupPage(id);
}

void NeuWizard::accept() {
    QWizard::accept();
    delete *oriFigur;
    *oriFigur = new Figur(actFigur);
}
